/*
 *
 * Copyright (c) 2001
 *	Politecnico di Torino.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foun on; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Author:	Marco Mellia, Andrea Carpani, Luca Muscariello, Dario Rossi
 * 		Telecomunication Networks Group
 * 		Politecnico di Torino
 * 		Torino, Italy
 *              http://http://www.tlc-networks.polito.it/index.html
 *		mellia@mail.tlc.polito.it
 *
 * Tstat is deeply based on TCPTRACE. The authors would like to thank
 * Shawn Ostermann for the development of TCPTRACE.
 *
*/

#include <stdio.h>
#include "tstat.h"

extern unsigned long int fcount;
extern Bool printticks;
#ifdef HAVE_RRDTOOL
/*----------------------------------------------------------*/
/*   RRDtools 				                    */
extern Bool pure_rrd_engine;
#endif
/* Pointer to the last valid histo */
struct double_histo_list *hl = NULL;

struct double_histo_list *
first_histo_list ()
{
  return hl;
}

/*
*   histo management
*/

void
create_all_histo (void)
{

  /* IP histograms */

  ip_protocol_in = create_histo ("ip_protocol_in",
				 "IP protocol - incoming packets", 0, 256, 1);
  ip_protocol_out = create_histo ("ip_protocol_out",
				  "IP protocol - outgoing packets", 0, 256,
				  1);
  ip_protocol_loc =
    create_histo ("ip_protocol_loc", "IP protocol - local packets", 0, 256,
		  1);

  ip_len_out = create_histo ("ip_len_out",
			     "IP packet length [byte] - outgoing packets", 0,
			     1600, 4);
  ip_len_in =
    create_histo ("ip_len_in", "IP packet length [byte] - incoming packets",
		  0, 1600, 4);
  ip_len_loc =
    create_histo ("ip_len_loc", "IP packet length [byte] - local packets", 0,
		  1600, 4);

  ip_ttl_out =
    create_histo ("ip_ttl_out", "IP TTL - outgoing packtes", 1, 257, 1);
  ip_ttl_in =
    create_histo ("ip_ttl_in", "IP TTL - incoming packets", 1, 257, 1);
  ip_ttl_loc =
    create_histo ("ip_ttl_loc", "IP TTL - local packets", 1, 257, 1);

  ip_tos_out =
    create_histo ("ip_tos_out", "IP TOS - outgoing packets", 1, 257, 1);
  ip_tos_in =
    create_histo ("ip_tos_in", "IP TOS - incoming packets", 1, 257, 1);
  ip_tos_loc =
    create_histo ("ip_tos_loc", "IP TOS - local packets", 1, 257, 1);


  /* TCP segment histograms */

  tcp_port_src_in = create_histo ("tcp_port_src_in",
				  "TCP source port - incoming segments", 0,
				  65536, 1);
  tcp_port_src_out = create_histo ("tcp_port_src_out",
				   "TCP source port - outgoing segments", 0,
				   65536, 1);
  tcp_port_src_loc = create_histo ("tcp_port_src_loc",
				   "TCP source port - local segments", 0,
				   65536, 1);
  tcp_port_dst_in = create_histo ("tcp_port_dst_in",
				  "TCP destination port - incoming segments",
				  0, 65536, 1);
  tcp_port_dst_out =
    create_histo ("tcp_port_dst_out",
		  "TCP destination port - outgoing segments", 0, 65536, 1);
  tcp_port_dst_loc =
    create_histo ("tcp_port_dst_loc", "TCP destination port - local segments",
		  0, 65536, 1);

  tcp_port_syn_src_in =
    create_histo ("tcp_port_syn_src_in",
		  "TCP source port of SYN segments - incoming segments", 0,
		  65536, 1);
  tcp_port_syn_src_out =
    create_histo ("tcp_port_syn_src_out",
		  "TCP source port of SYN segments - outgoing segments", 0,
		  65536, 1);
  tcp_port_syn_src_loc =
    create_histo ("tcp_port_syn_src_loc",
		  "TCP source port of SYN segments - local segments", 0,
		  65536, 1);
  tcp_port_syn_dst_in =
    create_histo ("tcp_port_syn_dst_in",
		  "TCP destination port of SYN segments - incoming segments",
		  0, 65536, 1);
  tcp_port_syn_dst_out =
    create_histo ("tcp_port_syn_dst_out",
		  "TCP destination port of SYN segments - outgoing segments",
		  0, 65536, 1);
  tcp_port_syn_dst_loc =
    create_histo ("tcp_port_syn_dst_loc",
		  "TCP destination port of SYN segments - local segments", 0,
		  65536, 1);

  /* TCP flow histograms */

  tcp_opts_SACK = create_histo ("tcp_opts_SACK", "TCP option: SACK", 1, 4, 1);
  tcp_opts_WS =
    create_histo ("tcp_opts_WS", "TCP option: WindowScale", 1, 4, 1);
  tcp_opts_TS =
    create_histo ("tcp_opts_TS", "TCP option: Timestamp", 1, 4, 1);

  tcp_mss_a =
    create_histo ("tcp_mss_a", "TCP declared client MSS [byte]", 0, 1600, 4);
  tcp_mss_b =
    create_histo ("tcp_mss_b", "TCP declared server MSS [byte]", 0, 1600, 4);
  tcp_mss_used =
    create_histo ("tcp_mss_used", "TCP negotiated MSS [byte]", 0, 1600, 4);

  tcp_win_min =
    create_histo ("tcp_win_ini", "TCP initial RWND [byte]", 0, 65536, 256);
  tcp_win_avg =
    create_histo ("tcp_win_avg", "TCP average RWND [byte]", 0, 65536, 256);
  tcp_win_max =
    create_histo ("tcp_win_max", "TCP max RWND [byte]", 0, 65536, 256);

  tcp_cwnd =
    create_histo ("tcp_cwnd", "TCP in-flight-size [byte]", 0, 65536, 256);

  tcp_cl_p_out =
    create_histo ("tcp_cl_p_out",
		  "TCP flow length [packet] - outgoing flows", 0, 1000, 1);

  tcp_cl_p_in =
    create_histo ("tcp_cl_p_in",
		  "TCP flow length [packet] - incoming flows", 0, 1000, 1);
  tcp_cl_p_loc =
    create_histo ("tcp_cl_p_loc",
		  "TCP flow length [packet] - local flows", 0, 1000, 1);

  tcp_cl_p_c2s =
    create_histo ("tcp_cl_p_c2s",
		  "TCP flow length [packet] - clientflows", 0, 1000, 1);

  tcp_cl_p_s2c =
    create_histo ("tcp_cl_p_s2c",
		  "TCP flow length [packet] - server flows", 0, 1000, 1);


  tcp_cl_b_s_out =
    create_histo ("tcp_cl_b_s_out",
		  "TCP flow length [byte] - fine granularity histogram - outgoing flows",
		  0, 50000, 50);
  tcp_cl_b_s_in =
    create_histo ("tcp_cl_b_s_in",
		  "TCP flow length [byte] - fine granularity histogram - incoming flows",
		  0, 50000, 50);
  tcp_cl_b_s_loc =
    create_histo ("tcp_cl_b_s_loc",
		  "TCP flow length [byte] - fine granularity histogram - local flows",
		  0, 50000, 50);

  tcp_cl_b_l_out =
    create_histo ("tcp_cl_b_l_out",
		  "TCP flow length [byte] - coarse granularity histogram - outgoing flows",
		  0, 50000000, 50000);

  tcp_cl_b_l_in =
    create_histo ("tcp_cl_b_l_in",
		  "TCP flow length [byte] - coarse granularity histogram - incoming flows",
		  0, 50000000, 50000);
  tcp_cl_b_l_loc =
    create_histo ("tcp_cl_b_l_loc",
		  "TCP flow length [byte] - coarse granularity histogram - local flows",
		  0, 50000000, 50000);

  tcp_cl_b_s_c2s =
    create_histo ("tcp_cl_b_s_c2s",
		  "TCP flow length [byte] - fine granularity histogram - client flows",
		  0, 50000, 50);

  tcp_cl_b_s_s2c =
    create_histo ("tcp_cl_b_s_s2c",
		  "TCP flow length [byte] - fine granularity histogram - server flows",
		  0, 50000, 50);
  tcp_cl_b_l_c2s =
    create_histo ("tcp_cl_b_l_c2s",
		  "TCP flow length [byte] - coarse granularity histogram - client flows",
		  0, 50000000, 50000);
  tcp_cl_b_l_s2c =
    create_histo ("tcp_cl_b_l_s2c",
		  "TCP flow length [byte] - coarse granularity histogram - server flows",
		  0, 50000000, 50000);

  tcp_rtt_min_out =
    create_histo ("tcp_rtt_min_out", "TCP flow minimum RTT [ms] - outgoing flows", 0,
		  3500, 10);
  tcp_rtt_min_in =
    create_histo ("tcp_rtt_min_in", "TCP flow minimum RTT [ms]- incoming flows", 0,
		  3500, 10);
  tcp_rtt_min_loc =
    create_histo ("tcp_rtt_min_loc", "TCP flow minimum RTT - local flows", 0,
		  3500, 10);
  tcp_rtt_min_c2s =
    create_histo ("tcp_rtt_min_c2s", "TCP flow minimum RTT [ms] - client flows", 0,
		  3500, 10);
  tcp_rtt_min_s2c =
    create_histo ("tcp_rtt_min_s2c", "TCP flow minimum RTT [ms] - server flows", 0,
		  3500, 10);

  tcp_rtt_avg_out =
    create_histo ("tcp_rtt_avg_out", "TCP flow average RTT [ms] - outgoing flows", 0,
		  3500, 10);
  tcp_rtt_avg_in =
    create_histo ("tcp_rtt_avg_in", "TCP flow average RTT [ms] - incoming flows", 0,
		  3500, 10);
  tcp_rtt_avg_loc =
    create_histo ("tcp_rtt_avg_loc", "TCP flow average RTT [ms] - local flows", 0,
		  3500, 10);
  tcp_rtt_avg_c2s =
    create_histo ("tcp_rtt_avg_c2s", "TCP flow average RTT [ms] - client flows", 0,
		  3500, 10);
  tcp_rtt_avg_s2c =
    create_histo ("tcp_rtt_avg_s2c", "TCP flow average RTT [ms] - server flows", 0,
		  3500, 10);

  tcp_rtt_max_out =
    create_histo ("tcp_rtt_max_out", "TCP flow maximum RTT [ms] - outgoing flows", 0,
		  3500, 10);
  tcp_rtt_max_in =
    create_histo ("tcp_rtt_max_in", "TCP flow maximum RTT [ms] - incoming flows", 0,
		  3500, 10);
  tcp_rtt_max_loc =
    create_histo ("tcp_rtt_max_loc", "TCP flow maximum RTT [ms] - local flows", 0,
		  3500, 10);
  tcp_rtt_max_c2s =
    create_histo ("tcp_rtt_max_c2s", "TCP flow maximum RTT [ms] - client flows", 0,
		  3500, 10);
  tcp_rtt_max_s2c =
    create_histo ("tcp_rtt_max_s2c", "TCP flow maximum RTT [ms] - server flows", 0,
		  3500, 10);

  tcp_rtt_stdev_out =
    create_histo ("tcp_rtt_stdev_out",
		  "TCP flow RTT standard deviation [ms] - outgoing flows", 0, 3500,
		  10);
  tcp_rtt_stdev_in =
    create_histo ("tcp_rtt_stdev_in",
		  "TCP flow RTT standard deviation [ms] - incoming flows", 0, 3500,
		  10);
  tcp_rtt_stdev_loc =
    create_histo ("tcp_rtt_stdev_loc",
		  "TCP flow RTT standard deviation [ms] - local flows", 0, 3500, 10);
  tcp_rtt_stdev_c2s =
    create_histo ("tcp_rtt_stdev_c2s",
		  "TCP flow RTT standard deviation [ms] - client flows", 0, 3500, 10);
  tcp_rtt_stdev_s2c =
    create_histo ("tcp_rtt_stdev_s2c",
		  "TCP flow RTT standard deviation [ms] - server flows", 0, 3500, 10);

  tcp_rtt_cnt_out = create_histo ("tcp_rtt_cnt_out",
				  "TCP flow RTT valid samples - outgoing flows",
				  0, 200, 1);
  tcp_rtt_cnt_in =
    create_histo ("tcp_rtt_cnt_in", "TCP flow RTT valid samples - incoming flows",
		  0, 200, 1);
  tcp_rtt_cnt_loc =
    create_histo ("tcp_rtt_cnt_loc", "TCP flow RTT valid samples - local flows",
		  0, 200, 1);
  tcp_rtt_cnt_c2s =
    create_histo ("tcp_rtt_cnt_c2s", "TCP flow RTT valid samples - client flows",
		  0, 200, 1);
  tcp_rtt_cnt_s2c =
    create_histo ("tcp_rtt_cnt_s2c", "TCP flow RTT valid samples - server flows",
		  0, 200, 1);

  tcp_anomalies_in =
    create_histo ("tcp_anomalies_in",
		  "TCP total number of anomalies - incoming flows", 0,
		  NUM_TCP_ANOMALIES, 1);
  tcp_anomalies_out =
    create_histo ("tcp_anomalies_out",
		  "TCP total number of anomalies - outgoing flows", 0,
		  NUM_TCP_ANOMALIES, 1);
  tcp_anomalies_loc =
    create_histo ("tcp_anomalies_loc",
		  "TCP total number of anomalies - local flows", 0,
		  NUM_TCP_ANOMALIES, 1);
  tcp_anomalies_c2s =
    create_histo ("tcp_anomalies_c2s",
		  "TCP total number of anomalies - client flows", 0,
		  NUM_TCP_ANOMALIES, 1);
  tcp_anomalies_s2c =
    create_histo ("tcp_anomalies_s2c",
		  "TCP total number of anomalies - server flows", 0,
		  NUM_TCP_ANOMALIES, 1);

  tcp_rtx_RTO_in = create_histo ("tcp_rtx_RTO_in",
				 "TCP Number of RTO Retransmission - incoming flows",
				 0, 100, 1);
  tcp_rtx_RTO_out =
    create_histo ("tcp_rtx_RTO_out",
		  "TCP Number of RTO Retransmission - outgoing flows", 0, 100, 1);
  tcp_rtx_RTO_loc =
    create_histo ("tcp_rtx_RTO_loc",
		  "TCP Number of RTO Retransmission - local flows", 0, 100, 1);
  tcp_rtx_RTO_c2s =
    create_histo ("tcp_rtx_RTO_c2s",
		  "TCP Number of RTO Retransmission - client flows", 0, 100, 1);
  tcp_rtx_RTO_s2c =
    create_histo ("tcp_rtx_RTO_s2c",
		  "TCP Number of RTO Retransmission - server flows", 0, 100, 1);

  tcp_rtx_FR_in = create_histo ("tcp_rtx_FR_in",
				"TCP number of FR Retransmission - incoming flows",
				0, 100, 1);
  tcp_rtx_FR_out =
    create_histo ("tcp_rtx_FR_out",
		  "TCP Number of FR Retransmission - outgoing flows", 0, 100, 1);
  tcp_rtx_FR_loc =
    create_histo ("tcp_rtx_FR_loc",
		  "TCP Number of FR Retransmission - local flows", 0, 100, 1);
  tcp_rtx_FR_c2s =
    create_histo ("tcp_rtx_FR_c2s",
		  "TCP Number of FR Retransmission - client flows", 0, 100, 1);
  tcp_rtx_FR_s2c =
    create_histo ("tcp_rtx_FR_s2c",
		  "TCP number of FR Retransmission - server flows", 0, 100, 1);

  tcp_reordering_in = create_histo ("tcp_reordering_in",
				    "TCP number of packet reordering - incoming flows",
				    0, 100, 1);
  tcp_reordering_out =
    create_histo ("tcp_reordering_out",
		  "TCP number of packet reordering - outgoing flows", 0, 100, 1);
  tcp_reordering_loc =
    create_histo ("tcp_reordering_loc",
		  "TCP number of packet reordering - local flows", 0, 100, 1);
  tcp_reordering_c2s =
    create_histo ("tcp_reordering_c2s",
		  "TCP number of packet reordering - client flows", 0, 100, 1);
  tcp_reordering_s2c =
    create_histo ("tcp_reordering_s2c",
		  "TCP number of packet reordering - server flows", 0, 100, 1);

  tcp_net_dup_in = create_histo ("tcp_net_dup_in",
				 "TCP number of Network duplicates - incoming flows",
				 0, 100, 1);
  tcp_net_dup_out =
    create_histo ("tcp_net_dup_out",
		  "TCP number of Network duplicates - outgoing flows", 0, 100, 1);
  tcp_net_dup_loc =
    create_histo ("tcp_net_dup_out",
		  "TCP number of Network duplicates - local flows", 0, 100, 1);
  tcp_net_dup_c2s =
    create_histo ("tcp_net_dup_c2s",
		  "TCP number of Network duplicates - client flows", 0, 100, 1);
  tcp_net_dup_s2c =
    create_histo ("tcp_net_dup_s2c",
		  "TCP number of Network duplicates - server flows", 0, 100, 1);

  tcp_unknown_in = create_histo ("tcp_unknown_in",
				 "TCP number of unknown anomalies - incoming flows",
				 0, 100, 1);
  tcp_unknown_out =
    create_histo ("tcp_unknown_out",
		  "TCP number of unknown anomalies - outgoing flows", 0, 100, 1);
  tcp_unknown_loc =
    create_histo ("tcp_unknown_loc",
		  "TCP number of unknown anomalies - local flows", 0, 100, 1);
  tcp_unknown_c2s =
    create_histo ("tcp_unknown_c2s",
		  "TCP number of unknown anomalies - client flows", 0, 100, 1);
  tcp_unknown_s2c =
    create_histo ("tcp_unknown_s2c",
		  "TCP number of unknown anomalies - server flows", 0, 100, 1);

  tcp_flow_control_in = create_histo ("tcp_flow_control_in",
				      "TCP number of Flow Control - incoming flows",
				      0, 100, 1);
  tcp_flow_control_out =
    create_histo ("tcp_flow_control_out",
		  "TCP number of Flow Control - outgoing flows", 0, 100, 1);
  tcp_flow_control_loc =
    create_histo ("tcp_flow_control_loc",
		  "TCP number of Flow Control - local flows", 0, 100, 1);
  tcp_flow_control_c2s =
    create_histo ("tcp_flow_control_c2s",
		  "TCP number of Flow Control - client flows", 0, 100, 1);
  tcp_flow_control_s2c =
    create_histo ("tcp_flow_control_s2c",
		  "TCP number of Flow Control - server flows", 0, 100, 1);

  tcp_unnecessary_rtx_RTO_in = create_histo ("tcp_unnecessary_rtx_RTO_in",
					     "TCP number of Unneeded RTO retransmission - incoming flows",
					     0, 100, 1);
  tcp_unnecessary_rtx_RTO_out =
    create_histo ("tcp_unnecessary_rtx_RTO_out",
		  "TCP number of Unneeded RTO retransmission - outgoing flows", 0,
		  100, 1);
  tcp_unnecessary_rtx_RTO_loc =
    create_histo ("tcp_unnecessary_rtx_RTO_loc",
		  "TCP number of Unneeded RTO retransmission - local flows", 0,
		  100, 1);
  tcp_unnecessary_rtx_RTO_c2s =
    create_histo ("tcp_unnecessary_rtx_RTO_c2s",
		  "TCP number of Unneeded RTO retransmission - client flows", 0,
		  100, 1);
  tcp_unnecessary_rtx_RTO_s2c =
    create_histo ("tcp_unnecessary_rtx_RTO_s2c",
		  "TCP number of Unneeded RTO retransmission - server flows", 0,
		  100, 1);

  tcp_unnecessary_rtx_FR_in = create_histo ("tcp_unnecessary_rtx_FR_in",
					    "TCP number of Unneeded FR retransmission - incoming flows",
					    0, 100, 1);
  tcp_unnecessary_rtx_FR_out =
    create_histo ("tcp_unnecessary_rtx_FR_out",
		  "TCP number of Unneeded FR retransmission - outgoing flows", 0,
		  100, 1);
  tcp_unnecessary_rtx_FR_loc =
    create_histo ("tcp_unnecessary_rtx_FR_loc",
		  "TCP number of Unneeded FR retransmission - local flows", 0,
		  100, 1);
  tcp_unnecessary_rtx_FR_c2s =
    create_histo ("tcp_unnecessary_rtx_FR_c2s",
		  "TCP number of Unneeded FR retransmission - client flows", 0,
		  100, 1);
  tcp_unnecessary_rtx_FR_s2c =
    create_histo ("tcp_unnecessary_rtx_FR_s2c",
		  "TCP number of Unneeded FR retransmission - server flows", 0,
		  100, 1);

  tcp_tot_time = create_histo ("tcp_tot_time",
			       "TCP flow lifetime [ms]", 0, 720000, 50);
  tcp_thru_c2s = create_histo ("tcp_thru_c2s",
			       "TCP throughput [Kbps] - client flows", 0,
			       1000, 1);
  tcp_thru_s2c =
    create_histo ("tcp_thru_s2c", "TCP throughput [Kbps] - server flows", 0,
		  1000, 1);

  tcp_interrupted =
    create_histo ("tcp_interrupted", "TCP Early interrupted flows", 0, 1, 1);



  /* UDP histograms */


  udp_cl_p_out = create_histo ("udp_cl_p_out",
			       "UDP flow length [packet] - outgoing", 0,
			       1000, 1);
  udp_cl_p_in =
    create_histo ("udp_cl_p_in", "UDP flow length [packet] - incoming", 0,
		  1000, 1);
  udp_cl_p_loc =
    create_histo ("udp_cl_p_loc", "UDP flow length [packet] - local", 0,
		  1000, 1);

  udp_cl_b_s_out = create_histo ("udp_cl_b_s_out",
				 "UDP flow length [byte] - outgoing", 0,
				 50000, 50);
  udp_cl_b_s_in = create_histo ("udp_cl_b_s_in",
				"UDP flow length [byte] - incoming", 0,
				50000, 50);
  udp_cl_b_s_loc = create_histo ("udp_cl_b_s_loc",
				 "UDP flow length [byte] - local", 0,
				 50000, 50);
  udp_cl_b_l_out =
    create_histo ("udp_cl_b_l_out", "UDP flow length [byte] - outgoing", 0,
		  50000000, 50000);
  udp_cl_b_l_in =
    create_histo ("udp_cl_b_l_in", "UDP flow length [byte] - incoming", 0,
		  50000000, 50000);
  udp_cl_b_l_loc =
    create_histo ("udp_cl_b_l_loc", "UDP flow length [byte] - local", 0,
		  50000000, 50000);

  udp_tot_time = create_histo ("udp_tot_time",
			       "UDP flow lifetime [ms]", 0, 720000, 50);

  udp_port_dst_in = create_histo ("udp_port_dst_in",
				  "UDP destination post - incoming packets",
				  0, 65536, 1);
  udp_port_dst_out =
    create_histo ("udp_port_dst_out",
		  "UDP destination port - outgoing packets", 0, 65536, 1);
  udp_port_dst_loc =
    create_histo ("udp_port_dst_loc",
		  "UDP destination port - local packets", 0, 65536, 1);
  udp_port_flow_dst =
    create_histo ("udp_port_flow_dst",
		  "UDP destination port per flow", 0, 65536, 1);


  /* RTP histograms */


  rtp_tot_time_in =
    create_histo ("rtp_tot_time_in",
		  "RTP flow lifetime [ms] - incoming flows", 0, 1800, 1);
  rtp_tot_time_out =
    create_histo ("rtp_tot_time_out",
		  "RTP flow lifetime [ms] - outgoing flows", 0, 1800, 1);
  rtp_tot_time_loc =
    create_histo ("rtp_tot_time_loc", "RTP flow lifetime [ms] - local flows",
		  0, 1800, 1);
  rtp_cl_p_out =
    create_histo ("rtp_cl_p_out", "RTP flow length [packet] - outgoing flows",
		  0, 50000, 10);
  rtp_cl_p_in =
    create_histo ("rtp_cl_p_in", "RTP flow length [packet] - incoming flows",
		  0, 50000, 10);
  rtp_cl_p_loc =
    create_histo ("rtp_cl_p_loc", "RTP flow length [packet] - local flows", 0,
		  50000, 10);
  rtp_avg_delay_in =
    create_histo ("rtp_avg_delay_in", "RTP average delay - incoming flows", 0,
		  5000, 1);
  rtp_avg_delay_out =
    create_histo ("rtp_avg_delay_out", "RTP average delay - outgoing flows",
		  0, 5000, 1);
  rtp_avg_delay_loc =
    create_histo ("rtp_avg_delay_loc", "RTP average delay - local flows", 0,
		  5000, 1);
  rtp_avg_jitter_in =
    create_histo ("rtp_avg_jitter_in", "RTP average jitter - incoming flows",
		  0, 5000, 1);
  rtp_avg_jitter_out =
    create_histo ("rtp_avg_jitter_out", "RTP average jitter - outgoing flows",
		  0, 5000, 1);
  rtp_avg_jitter_loc =
    create_histo ("rtp_avg_jitter_loc", "RTP average jitter - local flows", 0,
		  5000, 1);
  rtp_n_oos_in =
    create_histo ("rtp_n_oos_in",
		  "RTP number of  out-of-sequence packets - incoming flows ",
		  0, 100, 1);
  rtp_n_oos_out =
    create_histo ("rtp_n_oos_out",
		  "RTP number of  out-of-sequence packets - outgoing flows", 0,
		  100, 1);
  rtp_n_oos_loc =
    create_histo ("rtp_n_oos_loc",
		  "RTP number of  out-of-sequence packets - local flows", 0,
		  100, 1);
  rtp_p_oos_in =
    create_histo ("rtp_p_oos_in",
		  "RTP total number of  out-of-sequence packets - incoming flows",
		  0, 1000, 1);
  rtp_p_oos_out =
    create_histo ("rtp_p_oos_out",
		  "RTP total number of  out-of-sequence packets - outgoing flows",

		  0, 1000, 1);
  rtp_p_oos_loc =
    create_histo ("rtp_p_oos_loc",
		  "RTP percentage of  out-of-sequence packets local flows", 0,
		  1000, 1);
  rtp_p_dup_in =
    create_histo ("rtp_p_dup_in",
		  "RTP prob of duplicate packets - incoming flows ", 0, 1000,
		  1);
  rtp_p_dup_out =
    create_histo ("rtp_p_dup_out",
		  "RTP prob of duplicate packets - outgoing flows", 0, 1000,
		  1);
  rtp_p_dup_loc =
    create_histo ("rtp_p_dup_loc",
		  "RTP prob of duplicate packets - local flows", 0, 1000, 1);
  rtp_p_lost_in =
    create_histo ("rtp_p_lost_in",
		  "RTP prob of lost packets - incoming flows ", 0, 1000, 1);
  rtp_p_lost_out =
    create_histo ("rtp_p_lost_out",
		  "RTP prob of lost packets - outgoing flows", 0, 1000, 1);
  rtp_p_lost_loc =
    create_histo ("rtp_p_lost_loc", "RTP prob of lost packets - local flows",
		  0, 1000, 1);
  rtp_p_late_in =
    create_histo ("rtp_p_late_in",
		  "RTP prob of late packets - incoming flows ", 0, 1000, 1);
  rtp_p_late_out =
    create_histo ("rtp_p_late_out",
		  "RTP prob of late packets - outgoing flows", 0, 1000, 1);
  rtp_p_late_loc =
    create_histo ("rtp_p_late_loc", "RTP prob of late packets - local flows",
		  0, 1000, 1);
  rtp_burst_loss_in =
    create_histo ("rtp_burst_loss_in",
		  "RTP burst length [packet]- incoming flows ", 0, 20, 1);
  rtp_burst_loss_out =
    create_histo ("rtp_burst_loss_out",
		  "RTP burst length [packet] - outgoing flows", 0, 20, 1);
  rtp_burst_loss_loc =
    create_histo ("rtp_burst_loss_loc", "RTP burst length [packet] - local flows",
		  0, 20, 1);


  rtp_reord_p_n_in =
    create_histo ("rtp_reord_p_n_in",
		  "RTP number of reordered packets - incoming flows ", 0, 0, 1);
  rtp_reord_p_n_out =
    create_histo ("rtp_reord_p_n_out",
		  "RTP number of reordered packets - outgoing flows", 0, 0, 1);
  rtp_reord_p_n_loc =
    create_histo ("rtp_reord_p_n_loc",
		  "RTP number of reordered packets - local flows", 0, 0, 1);
  rtp_reord_delay_in =
    create_histo ("rtp_reord_delay_in",
		  "RTP delay of reordered packets - incoming flows ", 0, 100, 1);
  rtp_reord_delay_out =
    create_histo ("rtp_reord_delay_out",
		  "RTP delay of reordered packets - outgoing flows", 0, 100, 1);
  rtp_reord_delay_loc =
    create_histo ("rtp_reord_delay_loc",
		  "RTP delay of reordered packets - local flows", 0, 100, 1);
  rtp_oos_p_in =
    create_histo ("rtp_oos_p_in",
		  "RTP number of  out of sequence packets - incoming flows", 0,
		  0, 1);
  rtp_oos_p_out =
    create_histo ("rtp_oos_p_out",
		  "RTP number of  out of sequence packets - outgoing flows", 0,
		  0, 1);
  rtp_oos_p_loc =
    create_histo ("rtp_oos_p_loc",
		  "RTP number of  out of sequence packets - local flows", 0, 0,
		  1);


  /* RTCP histograms */

  rtcp_cl_p_out = create_histo ("rtcp_cl_p_out",
				"RTCP flow length [packet] - outgoing flows",
				0, 3000, 1);
  rtcp_cl_p_in = create_histo ("rtcp_cl_p_in",
			       "RTCP flow length [packet] - incoming flows",
			       0, 3000, 1);
  rtcp_cl_p_loc = create_histo ("rtcp_cl_p_loc",
				"RTCP flow length [packet] - local flows",
				0, 3000, 1);
  rtcp_avg_inter_in =
    create_histo ("rtcp_avg_inter_delay_in",
		  "RTCP interarrival delay - incoming flows", 0, 5000, 1);
  rtcp_avg_inter_out =
    create_histo ("rtcp_avg_inter_delay_out",
		  "RTCP interarrival delay - outgoing flows", 0, 5000, 1);
  rtcp_avg_inter_loc =
    create_histo ("rtcp_avg_inter_delay_loc",
		  "RTCP interarrival delay - local flows", 0, 5000, 1);

  /* MISC histograms */

  flow_number =
    create_histo ("flow_number",
		  "Number of tracked TCP/UDP/RTP/RTCP flow", 0,
		  HISTO_FLOW_TOT, 1);
}

/* to speed up the code, try to make add_histo inlined */
inline void
add_histo (struct double_histo_list *p, double val)
{

  int i;
#ifdef HAVE_RRDTOOL
  double oldavg = p->st.avg;
#endif

  if(isnan(val) ||  isinf(val)) return;
  // if(!finite(val)) return;
  
#ifdef HAVE_RRDTOOL
  p->st.count++;
  p->st.avg += (val - oldavg) / p->st.count;
  /* var and stdev are calculated on the fly */
  p->st.pseudovar += (val - oldavg) * (val - p->st.avg);
  if (val > p->st.max)
    p->st.max = val;
  if (val < p->st.min)
    p->st.min = val;

#endif

  /* val < min ? put it in the first bin */
  if (val < p->min)
    {
      p->current_data[0]++;
    }
  else
  /* val > max ? put it in the last bin */
  if (val >= p->max)
    {
      p->current_data[(int) ((p->max - p->min) / p->bin_size + 1)]++;
    }
  else
  {
    i = ((int) val + p->bin_size - p->min) / (int) p->bin_size;
    p->current_data[i]++;

    /* count the total number of hits */
    p->current_data[(int) ((p->max - p->min) / p->bin_size + 2)]++;
  }
}


struct double_histo_list *
create_histo (char *name, char *descr, long min, long max, long bin_size)
{
  struct double_histo_list *thisto;
  int num_col;
  struct double_histo_list *temphp;

  num_col = ((max - min) / bin_size);

  if (max != (min + (num_col * bin_size)))
    {
      printf ("Error: wrong 'bin_size' param when creating %s histogram \n",
	      name);
      return 0;
    }

  thisto =
    (struct double_histo_list *) MMmalloc (sizeof (struct double_histo_list),
					   "create_histo");
  strncpy (thisto->name, name, HISTO_NAME_MAXLEN);
  strncpy (thisto->descr, descr, HISTO_DESCR_MAXLEN);
  thisto->min = min;
  thisto->max = max;
  thisto->bin_size = bin_size;

  /* histo[0] will count all samples smaller than min */
  /* histo[ 1 ... num_col-1 ] will count all samples between min and max
     with a step equal to dealta */
  /* histo[num_col] will count all samples larger than max */
  /* histo[num_col+1] will count all samples */

  thisto->first_data =
    (long *) MMmalloc (sizeof (long) * (num_col + 3), "create_histo");
  thisto->second_data =
    (long *) MMmalloc (sizeof (long) * (num_col + 3), "create_histo");

  /* add this histogram to the global list */
  temphp =
    (struct double_histo_list *) MMmalloc (sizeof (struct double_histo_list),
					   "create_histo");
  thisto->current_data = thisto->first_data;

  thisto->next = hl;
  hl = thisto;

  if (debug > 3)
    fprintf (stderr,
	     "Created histogram %s: [%ld:%ld], %d bins, %ld bin size\n", name,
	     min, max, num_col, bin_size);

  return thisto;
}

void
swap_histo ()
{
  struct double_histo_list *temphp;

  temphp = hl;
  while (temphp != NULL)
    {
      if (temphp->current_data == temphp->first_data)
	temphp->current_data = temphp->second_data;
      else
	temphp->current_data = temphp->first_data;

      temphp = temphp->next;
    }

#ifdef DEBUG_THREAD
  printf ("All histos swapped!\n");
#endif

}

/* Return the pointer to the unused histo */
long *
whats_frozen (struct double_histo_list *isto)
{

  if (isto->current_data == isto->first_data)
    return (isto->second_data);
  else
    return (isto->first_data);
}


int
clear_all_histo ()
{
  struct double_histo_list *temphp;

  temphp = hl;
  while (temphp != NULL)
    {
      clear_histo (temphp);
      temphp = temphp->next;
    }
  return 1;
}

int
clear_histo (struct double_histo_list *phisto)
{
  int num_col, i;
  long *frozen;

  num_col = (((phisto->max) - (phisto->min)) / phisto->bin_size) + 3;
  frozen = whats_frozen (phisto);
  for (i = 0; i < num_col; i++)
    {
      frozen[i] = 0;
    }
  return 1;
}


int
print_all_histo_definition ()
{
  struct double_histo_list *temphp;

  printf ("#name\t\t|min\t|bin_size\t|max\t|description\n");
  temphp = hl;
  while (temphp != NULL)
    {
      print_histo_definition (temphp, temphp->name);
      temphp = temphp->next;
    }
  return 1;
}

void
print_histo_definition (struct double_histo_list *phisto, char *titolo)
{
  printf ("#%s\t|", phisto->name);
  printf ("%ld\t|%ld\t|%ld\t|", phisto->min, phisto->bin_size, phisto->max);
  printf ("#%s\n", phisto->descr);
}

int
print_all_histo ()
{
  struct double_histo_list *temphp;

  temphp = hl;
  while (temphp != NULL)
    {
#ifdef HAVE_RRDTOOL
      /*---------------------------------------------------------*/
      /* RRDtools                                                */

      rrdtool_update (temphp);

      /*---------------------------------------------------------*/
      if (!pure_rrd_engine)
#endif
	print_histo (temphp, temphp->name, 1);
      temphp = temphp->next;
    }
  return 1;
}

int
print_histo (struct double_histo_list *phisto, char *titolo, int flag)
{
  long min, max, bin_size;
  int num_row, i;
  struct stat fbuf;
  FILE *fp;
  char filename[200] = "";
  long *frozen;

  min = phisto->min;
  max = phisto->max;
  bin_size = phisto->bin_size;

  /* check directory status */
  if (stat (curr_data_dir, &fbuf) == -1)
    {
      if (printticks)
	fprintf (stderr, "\rCreating output dir %s              \n",
		 curr_data_dir);
      else
	fprintf (stderr, "Creating output dir %s\n", curr_data_dir);
      mkdir (curr_data_dir, 0775);
    }

  sprintf (filename, "%s/%s", curr_data_dir, titolo);
  fp = fopen (filename, "w");

  if (fp == NULL)
    {
      if (printticks)
	fprintf (stderr, "\rCould not open file %s\n", filename);
      else
	fprintf (stderr, "Could not open file %s\n", filename);
      return 0;
    }

  /* how many rows? +2 to include (premin e postmax) */
  num_row = ((max - min) / bin_size) + 2;

  /* print the frozen data */
  frozen = whats_frozen (phisto);

  /* Flag = 1 print start bin
     Flag = 0 if you do not want them, but rather the row number
   */

  fprintf (fp, "#%s\n", phisto->descr);
  fprintf (fp, "#min=%ld bin_size=%ld max=%ld\n", min, bin_size, max);

  for (i = 0; i < num_row; i++)
    {
      if (flag)			/* print correct index */
	{
	  if (frozen[i] != 0)	/* just skip zeros to compact this */
	    fprintf (fp, "%ld %ld\n", (min + (i - 1) * bin_size), frozen[i]);
	}
      else			/* print internal index */
	{
	  if (frozen[i] != 0)	/* just skip zeros to compact this */
	    fprintf (fp, "%d %ld\n", i, frozen[i]);
	}
    }
  fclose (fp);
  return 1;
}

/* not used, but may be usefull in the future... */

int
print_text_histo (struct double_histo_list *phisto, char *titolo,
		  char *labels)
{
  double min, max, bin_size;
  int num_col, i;
  char *llabels;
  long *frozen;

  min = phisto->min;
  max = phisto->max;
  bin_size = phisto->bin_size;
  llabels = labels;

  /* how many rows? +2 to include (premin e postmax) */
  num_col = ((max - min) / bin_size) + 2;

  printf ("\nResults for [%f:%f] with %d steps (%f)\n", min, max, num_col,
	  bin_size);
  printf ("*** %s ***\n", titolo);

  /* print the frozen data */
  frozen = whats_frozen (phisto);

  for (i = 0; i < num_col; i++)
    {
      /* stampo la riga */
      while ((*llabels != '!') && (*llabels != '\0'))
	{
	  printf ("%c", *llabels);
	  llabels++;
	}
      if (*llabels != '\0')
	{
	  llabels++;
	}
      else
	{
	  printf ("Val");
	}
      printf ("\t:%ld\n", frozen[i]);
    }
  return 1;
}

/*
*  dynamic histo 
*/

int
print_dhisto (struct dhisto *phisto, char *titolo, char *labels)
{
  struct dato *elem;
  double min, max, bin_size;
  int num_col, i;
  char *llabels;


  min = phisto->min;
  max = phisto->max;
  bin_size = phisto->bin_size;
  elem = phisto->data;
  llabels = labels;

  num_col = ((max - min) / bin_size) + 2;

  printf ("\nResults for [%f:%f] with %d steps (%f)\n", min, max, num_col,
	  bin_size);
  printf ("*** %s ***\n", titolo);
  for (i = 1; i <= num_col; i++)
    {
      while ((*llabels != '!') && (*llabels != '\0'))
	{
	  printf ("%c", *llabels);
	  llabels++;
	}
      if (*llabels != '\0')
	{
	  llabels++;
	}
      else
	{
	  printf ("Val");
	}
      printf (":\t%d\n", elem->val);
      elem = elem->next;
    }
  return 1;
}

int
add_dhisto (struct dhisto *phisto, double val)
{
  double min, max, bin_size;
  int num_col, i, j;
  struct dato *elem;

  min = phisto->min;
  max = phisto->max;
  bin_size = phisto->bin_size;
  elem = phisto->data;

  num_col = (max - min) / bin_size;

  if (val < min)
    {
      elem->val++;
      return 1;
    }

  for (i = 1; i <= num_col; i++)
    {
      if (val <= (min + (bin_size * i)))
	{
	  elem = phisto->data;
	  for (j = 1; j <= i; j++)
	    {
	      elem = elem->next;
	    }
	  elem->val++;
	  return 1;
	}
    }

  elem = phisto->data;

  elem = elem->next;
  for (j = 1; j < num_col; j++)
    {
      elem = elem->next;
    }
  elem = elem->next;

  elem->val++;
  return 1;
}

struct dhisto *
create_dhisto2 (double min, double max, int num_col)
{
  double bin_size;
  struct dhisto *htemp;

  bin_size = (max - min) / num_col;

  htemp = create_dhisto (min, max, bin_size);

  if (!htemp)
    {
      return 0;
    }
  else
    {
      return htemp;
    }
}

struct dhisto *
create_dhisto (double min, double max, double bin_size)
{
  struct dhisto *htemp;
  struct dato *elem, *temp_dato;
  int i, num_col;

  htemp = (struct dhisto *) malloc (sizeof (struct dhisto));
  htemp->min = min;
  htemp->max = max;
  htemp->bin_size = bin_size;
  htemp->data = NULL;

  elem = htemp->data;

  num_col = ((max - min) / bin_size);

  if (max != (min + (num_col * bin_size)))
    {
      printf ("Wrong bin_size=%f values!\n", bin_size);
      return 0;
    }


  /* Recall that we use 2 bins to store values smaller than the min and
   * larger than the max limit
   */

  /* The first one is here */
  temp_dato = (struct dato *) malloc (sizeof (struct dato));
  temp_dato->val = 0;
  temp_dato->next = NULL;
  elem = temp_dato;
  htemp->data = temp_dato;


  /* ... now add the remaining bins */

  for (i = 0; i <= num_col; i++)
    {

      /* Go to the last bin */
      while (elem->next != NULL)
	{
	  elem = elem->next;
	}

      /* add a data to the tail */
      temp_dato = (struct dato *) malloc (sizeof (struct dato));
      temp_dato->val = 0;
      temp_dato->next = NULL;
      elem->next = temp_dato;

    }

  return htemp;
}
