/*
 *
 * Copyright (c) 2001
 *	Politecnico di Torino.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Author:	Marco Mellia, Andrea Carpani, Luca Muscariello, Dario Rossi
 * 		Telecomunication Networks Group
 * 		Politecnico di Torino
 * 		Torino, Italy
 *              http://http://www.tlc-networks.polito.it/index.html
 *		mellia@mail.tlc.polito.it
 *
 * Tstat is deeply based on TCPTRACE. The authors would like to thank
 * Shawn Ostermann for the development of TCPTRACE.
 *
*/
#define HISTO_NAME_MAXLEN 30
#define HISTO_DESCR_MAXLEN 256

struct dhisto
{
  double min;
  double max;
  double bin_size;
  struct dato *data;
};

struct dato
{
  int val;
  struct dato *next;
};

struct vhisto
{
  double min;
  double max;
  double bin_size;
  struct vdato *data;
};

/****************aste************/
struct double_histo_list
{
  char name[HISTO_NAME_MAXLEN];
  char descr[HISTO_DESCR_MAXLEN];
#ifdef HAVE_RRDTOOL
/*-----------------------------------------------------------*/
/* RRDtools 				                     */
  struct rrdstat st;
  struct rrdconf conf;
/*-----------------------------------------------------------*/
#endif
  long min;
  long max;
  long bin_size;
  /* data[0] will count all samples smaller than min */
  /* data[ 1 ... num_col-1 ] will count all samples between min and max
     with a step equal to dealta */
  /* data[num_col] will count all samples larger than max */
  /* data[num_col+1] will count all samples */
  long *first_data;
  long *second_data;
  long *current_data;
  struct double_histo_list *next;
};
/*********************************/


struct adx
{
  struct in_addr ip;
  unsigned long src_hits;
  unsigned long dst_hits;
  struct adx *next;
};

/* here are all the histo tracked by tstat */
/* struct double_histo_list *test; */
struct double_histo_list *ip_protocol_in;
struct double_histo_list *ip_protocol_out;
struct double_histo_list *ip_protocol_loc;
struct double_histo_list *tcp_port_src_in;
struct double_histo_list *tcp_port_src_out;
struct double_histo_list *tcp_port_src_loc;
struct double_histo_list *tcp_port_dst_in;
struct double_histo_list *tcp_port_dst_out;
struct double_histo_list *tcp_port_dst_loc;

struct double_histo_list *tcp_port_syn_src_in;
struct double_histo_list *tcp_port_syn_src_out;
struct double_histo_list *tcp_port_syn_src_loc;
struct double_histo_list *tcp_port_syn_dst_in;
struct double_histo_list *tcp_port_syn_dst_out;
struct double_histo_list *tcp_port_syn_dst_loc;


struct double_histo_list *tcp_opts_SACK;
struct double_histo_list *tcp_opts_WS;
struct double_histo_list *tcp_opts_TS;

struct double_histo_list *tcp_mss_a;
struct double_histo_list *tcp_mss_b;
struct double_histo_list *tcp_mss_used;

struct double_histo_list *tcp_win_min;
struct double_histo_list *tcp_win_avg;
struct double_histo_list *tcp_win_max;

struct double_histo_list *tcp_cwnd;

struct double_histo_list *tcp_cl_b_l_out;
struct double_histo_list *tcp_cl_b_l_in;
struct double_histo_list *tcp_cl_b_l_loc;
struct double_histo_list *tcp_cl_b_s_out;
struct double_histo_list *tcp_cl_b_s_in;
struct double_histo_list *tcp_cl_b_s_loc;
struct double_histo_list *tcp_cl_p_out;
struct double_histo_list *tcp_cl_p_in;
struct double_histo_list *tcp_cl_p_loc;

struct double_histo_list *tcp_cl_b_l_c2s;
struct double_histo_list *tcp_cl_b_l_s2c;
struct double_histo_list *tcp_cl_b_s_c2s;
struct double_histo_list *tcp_cl_b_s_s2c;
struct double_histo_list *tcp_cl_p_c2s;
struct double_histo_list *tcp_cl_p_s2c;

struct double_histo_list *ip_len_out;
struct double_histo_list *ip_len_in;
struct double_histo_list *ip_len_loc;
struct double_histo_list *ip_ttl_out;
struct double_histo_list *ip_ttl_in;
struct double_histo_list *ip_ttl_loc;
struct double_histo_list *ip_tos_out;
struct double_histo_list *ip_tos_in;
struct double_histo_list *ip_tos_loc;

struct double_histo_list *tcp_rtt_min_out;
struct double_histo_list *tcp_rtt_min_in;
struct double_histo_list *tcp_rtt_min_loc;
struct double_histo_list *tcp_rtt_min_c2s;
struct double_histo_list *tcp_rtt_min_s2c;

struct double_histo_list *tcp_rtt_avg_out;
struct double_histo_list *tcp_rtt_avg_in;
struct double_histo_list *tcp_rtt_avg_loc;
struct double_histo_list *tcp_rtt_avg_c2s;
struct double_histo_list *tcp_rtt_avg_s2c;

struct double_histo_list *tcp_rtt_max_out;
struct double_histo_list *tcp_rtt_max_in;
struct double_histo_list *tcp_rtt_max_loc;
struct double_histo_list *tcp_rtt_max_c2s;
struct double_histo_list *tcp_rtt_max_s2c;

struct double_histo_list *tcp_rtt_stdev_out;
struct double_histo_list *tcp_rtt_stdev_in;
struct double_histo_list *tcp_rtt_stdev_loc;
struct double_histo_list *tcp_rtt_stdev_c2s;
struct double_histo_list *tcp_rtt_stdev_s2c;

struct double_histo_list *tcp_rtt_cnt_out;
struct double_histo_list *tcp_rtt_cnt_in;
struct double_histo_list *tcp_rtt_cnt_loc;
struct double_histo_list *tcp_rtt_cnt_c2s;
struct double_histo_list *tcp_rtt_cnt_s2c;

struct double_histo_list *tcp_anomalies_c2s;
struct double_histo_list *tcp_anomalies_s2c;
struct double_histo_list *tcp_anomalies_out;
struct double_histo_list *tcp_anomalies_in;
struct double_histo_list *tcp_anomalies_loc;

struct double_histo_list *tcp_rtx_RTO_c2s;
struct double_histo_list *tcp_rtx_RTO_s2c;
struct double_histo_list *tcp_rtx_RTO_out;
struct double_histo_list *tcp_rtx_RTO_in;
struct double_histo_list *tcp_rtx_RTO_loc;

struct double_histo_list *tcp_rtx_FR_c2s;
struct double_histo_list *tcp_rtx_FR_s2c;
struct double_histo_list *tcp_rtx_FR_out;
struct double_histo_list *tcp_rtx_FR_in;
struct double_histo_list *tcp_rtx_FR_loc;

struct double_histo_list *tcp_reordering_c2s;
struct double_histo_list *tcp_reordering_s2c;
struct double_histo_list *tcp_reordering_out;
struct double_histo_list *tcp_reordering_in;
struct double_histo_list *tcp_reordering_loc;

struct double_histo_list *tcp_net_dup_c2s;
struct double_histo_list *tcp_net_dup_s2c;
struct double_histo_list *tcp_net_dup_out;
struct double_histo_list *tcp_net_dup_in;
struct double_histo_list *tcp_net_dup_loc;

struct double_histo_list *tcp_unknown_c2s;
struct double_histo_list *tcp_unknown_s2c;
struct double_histo_list *tcp_unknown_out;
struct double_histo_list *tcp_unknown_in;
struct double_histo_list *tcp_unknown_loc;

struct double_histo_list *tcp_flow_control_c2s;
struct double_histo_list *tcp_flow_control_s2c;
struct double_histo_list *tcp_flow_control_out;
struct double_histo_list *tcp_flow_control_in;
struct double_histo_list *tcp_flow_control_loc;

struct double_histo_list *tcp_unnecessary_rtx_RTO_c2s;
struct double_histo_list *tcp_unnecessary_rtx_RTO_s2c;
struct double_histo_list *tcp_unnecessary_rtx_RTO_out;
struct double_histo_list *tcp_unnecessary_rtx_RTO_in;
struct double_histo_list *tcp_unnecessary_rtx_RTO_loc;

struct double_histo_list *tcp_unnecessary_rtx_FR_c2s;
struct double_histo_list *tcp_unnecessary_rtx_FR_s2c;
struct double_histo_list *tcp_unnecessary_rtx_FR_out;
struct double_histo_list *tcp_unnecessary_rtx_FR_in;
struct double_histo_list *tcp_unnecessary_rtx_FR_loc;


struct double_histo_list *tcp_tot_time;
struct double_histo_list *tcp_thru_c2s;
struct double_histo_list *tcp_thru_s2c;

struct double_histo_list *tcp_interrupted;

struct double_histo_list *flow_number;


    /* istogrammi per udp */


struct double_histo_list *udp_cl_p_out;
struct double_histo_list *udp_cl_p_in;
struct double_histo_list *udp_cl_p_loc;

struct double_histo_list *udp_cl_b_l_out;
struct double_histo_list *udp_cl_b_l_in;
struct double_histo_list *udp_cl_b_l_loc;
struct double_histo_list *udp_cl_b_s_out;
struct double_histo_list *udp_cl_b_s_in;
struct double_histo_list *udp_cl_b_s_loc;

struct double_histo_list *udp_tot_time;
struct double_histo_list *udp_port_dst_in;
struct double_histo_list *udp_port_dst_out;
struct double_histo_list *udp_port_dst_loc;
struct double_histo_list *udp_port_flow_dst;


      /* istogrammi per rtp */

struct double_histo_list *rtp_tot_time_in;
struct double_histo_list *rtp_tot_time_out;
struct double_histo_list *rtp_tot_time_loc;

struct double_histo_list *rtp_cl_p_out;
struct double_histo_list *rtp_cl_p_in;
struct double_histo_list *rtp_cl_p_loc;
struct double_histo_list *rtp_avg_delay_in;
struct double_histo_list *rtp_avg_delay_out;
struct double_histo_list *rtp_avg_delay_loc;
struct double_histo_list *rtp_avg_jitter_in;
struct double_histo_list *rtp_avg_jitter_out;
struct double_histo_list *rtp_avg_jitter_loc;
struct double_histo_list *rtp_n_oos_in;
struct double_histo_list *rtp_n_oos_out;
struct double_histo_list *rtp_n_oos_loc;
struct double_histo_list *rtp_p_oos_in;
struct double_histo_list *rtp_p_oos_out;
struct double_histo_list *rtp_p_oos_loc;
struct double_histo_list *rtp_p_dup_in;
struct double_histo_list *rtp_p_dup_out;
struct double_histo_list *rtp_p_dup_loc;
struct double_histo_list *rtp_p_lost_in;
struct double_histo_list *rtp_p_lost_out;
struct double_histo_list *rtp_p_lost_loc;
struct double_histo_list *rtp_p_late_in;
struct double_histo_list *rtp_p_late_out;
struct double_histo_list *rtp_p_late_loc;
struct double_histo_list *rtp_burst_loss_in;
struct double_histo_list *rtp_burst_loss_out;
struct double_histo_list *rtp_burst_loss_loc;
struct double_histo_list *rtp_reord_p_n_in;
struct double_histo_list *rtp_reord_p_n_out;
struct double_histo_list *rtp_reord_p_n_loc;
struct double_histo_list *rtp_reord_delay_in;
struct double_histo_list *rtp_reord_delay_out;
struct double_histo_list *rtp_reord_delay_loc;
struct double_histo_list *rtp_oos_p_in;
struct double_histo_list *rtp_oos_p_out;
struct double_histo_list *rtp_oos_p_loc;
      /* istogrammi per rtcp */

struct double_histo_list *rtcp_cl_p_out;
struct double_histo_list *rtcp_cl_p_in;
struct double_histo_list *rtcp_cl_p_loc;
struct double_histo_list *rtcp_avg_inter_in;
struct double_histo_list *rtcp_avg_inter_out;
struct double_histo_list *rtcp_avg_inter_loc;

/* prototypes of functions */

struct double_histo_list *first_histo_list ();
void create_all_histo (void);
void add_histo (struct double_histo_list *p, double val);
struct dhisto *create_dhisto (double min, double max, double bin_size);
struct dhisto *create_dhisto2 (double min, double max, int num_col);
int add_dhisto (struct dhisto *phisto, double val);
int print_dhisto (struct dhisto *phisto, char *titolo, char *labels);

struct histo *create_vhisto (char *formato);

struct double_histo_list *create_histo (char *name, char *descr, long min,
					long max, long bin_size);

void swap_histo ();
long *whats_frozen (struct double_histo_list *histo);

int print_all_histo ();
int print_all_histo_definition ();
int print_histo (struct double_histo_list *phisto, char *titolo, int flag);
void print_histo_definition (struct double_histo_list *phisto, char *title);
int print_text_histo (struct double_histo_list *phisto, char *titolo,
		      char *labels);

int clear_all_histo ();
int clear_histo (struct double_histo_list *phisto);

int add_adx (struct in_addr *adx, int dir);
int print_adx ();
