/*
 *
 * Copyright (c) 2001
 *	Politecnico di Torino.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Author:	Marco Mellia, Andrea Carpani, Luca Muscariello, Dario Rossi
 * 		Telecomunication Networks Group
 * 		Politecnico di Torino
 * 		Torino, Italy
 *              http://http://www.tlc-networks.polito.it/index.html
 *		mellia@mail.tlc.polito.it
 *
 * Tstat is deeply based on TCPTRACE. The authors would like to thank
 * Shawn Ostermann for the development of TCPTRACE.
 *
*/

#ifdef HAVE_RRDTOOL

#include "tstat.h"

#define rrdtool_freeargv(ARGC,ARGV) int C; for(C=0; C<ARGC; C++) free(ARGV[C]);
void rrdtool_str2argv (char *buf, int *pargc, char ***pargv);
void rrdtool_init ();
void rrdtool_parse_config ();
void rrdtool_parse_line (char *line);
void rrdtool_update_all ();
void rrdtool_create_all ();
void rrdtool_update (struct double_histo_list *h);
void rrdtool_create (struct double_histo_list *h);
void rrdtool_update_command (const char *name, const char *what, double val);
void rrdtool_create_command (const char *name, const char *what);
void histo_parse_rrdconf (void);

/* these two are taken from rrd_tool.c */
int rrdtool_count_args (char *aLine);
int rrdtool_create_args (char *pName, char *aLine, int argc, char **argv);
struct double_histo_list *find_histo (char *name);

typedef struct _rrd
{
  struct _conf
  {
    char path[1024];		/* of the rrd files to be created */
    char file[1024];		/* of Tstat/RRDtool configuration file */
    int line;			/* of the Tstat/RRDtool configuration file */
  } conf;
  char cmd[4096];		/* for system() calls */
  char buf[4096];		/* to read files/pipes lines */
  char file[1024];		/* the rrdfile we will be working on:
				   sprintf("%s/%s.%s",rrd.conf.path,histo->name,statistical_var) */

  int fake_update;		/* so that ``update'' actually quit after ``creates'' */
  int fatal;			/* whether rrd errors are fatal */
  struct stat fbuf;
  int status;
  FILE *pipe;
} rrd_struct;

rrd_struct rrd;


/*============================================================================*
  rrdtool_str2argv
*-----------------------------------------------------------------------------*/
void
rrdtool_str2argv (char *buf, int *pargc, char ***pargv)
{
  char **argv;
  int nargs = 0;

  /* discard the original string, use a copy */
  buf = strdup (buf);

  /* (very pessimistically) make the argv array */
  argv = malloc (sizeof (char *) * ((strlen (buf) / 2) + 1));

  /* skip leading blanks */
  while ((*buf != '\00') && (isspace ((int) *buf)))
    {
      if (debug > 10)
	printf ("skipping isspace('%c')\n", *buf);
      ++buf;
    }

  argv[0] = '\0';
  /* break into args */
  for (nargs = 0; *buf != '\00'; ++nargs)
    {
      char *stringend;
      argv[nargs] = buf;

      /* search for separator */
      while ((*buf != '\00') && (!isspace ((int) *buf)))
	{
	  if (debug > 10)
	    printf ("'%c' (%d) is NOT a space\n", *buf, (int) *buf);
	  ++buf;
	}
      stringend = buf;

      /* skip spaces */
      while ((*buf != '\00') && (isspace ((int) *buf)))
	{
	  if (debug > 10)
	    printf ("'%c' (%d) IS a space\n", *buf, (int) *buf);
	  ++buf;
	}

      *stringend = '\00';	/* terminate the previous string */

      if (debug > 10)
	printf ("  argv[%d] = '%s'\n", nargs, argv[nargs]);
    }

  *pargc = nargs;
  *pargv = argv;
}


/*============================================================================*
  rrdtool_set_{conf,path}  
  rrdtool_init()
*-----------------------------------------------------------------------------*/

void
rrdtool_set_conf (char *file)
{
  sprintf (rrd.conf.file, "%s", file);
}

void
rrdtool_set_path (char *path)
{
  sprintf (rrd.conf.path, "%s", path);
}

void
rrdtool_init ()
{
  rrd_clear_error ();
  rrd.fatal = 1;
  rrdtool_parse_config ();
  /* we rather want to create this now... */
  /* rrdtool_create_all(); */
}


/*============================================================================*
	parse configurations
*-----------------------------------------------------------------------------*/
void
rrdtool_parse_config (void)
{
  FILE *conf;

  if (debug)
    fprintf (stderr, "rrdtool: delayed-parsing of config file <%s>\n",
	     rrd.conf.file);
  if (!(conf = fopen (rrd.conf.file, "r")))
    {
      fprintf (stderr, "%s: file open error.\n", rrd.conf.file);
      perror (NULL);
      exit (1);
    }

  while (!feof (conf))
    {
      fgets (rrd.buf, 1024, conf);
      rrd.conf.line++;
      if (index (rrd.buf, '#') == NULL)
	rrdtool_parse_line (rrd.buf);

      /* if(index(rrd.buf,"#") != NULL) {
         if (debug) fprintf(stderr, "\rignoring line %d (%s)", rrd.conf.line, rrd.buf);
         } else {
         if (debug) fprintf(stderr, "\rparsing line %d (%s)", rrd.conf.line, rrd.buf);            
         rrdtool_parse_line(rrd.buf);
         } */
    }
  if (debug)
    fprintf (stderr, "\rrrdtool: config file succesfully parsed (%d lines)\n",
	     rrd.conf.line);
}


/* shitty C makes it harder than perl */
void
rrdtool_parse_line (char *line)
{
  int iidx = 0, fidx = 0;
  char str[1024];
  int ivec[256];
  double fvec[256];
  char name[1024];
  struct double_histo_list *histo;
  char *pstr;
  /* find the histo named name (i.e., the first arg on the line) */

  char idxStr[1024] = "";
  char prcStr[1024] = "";

  if (debug)
    fprintf (stderr, "Parsing line %3d: ", rrd.conf.line);
  pstr = strtok (line, " ");
  sscanf (pstr, "%s", name);
  histo = find_histo (name);
  if (histo == NULL)
    {
      fprintf (stderr,
	       "rrdtool: variable <%s> unknown while processing <%s>\n", name,
	       rrd.conf.file);
      exit (1);
    }
  else if (debug)
    {
      fprintf (stderr, "%s\t", name);
    }


  histo->conf.rrd = TRUE;
  while ((pstr = strtok (NULL, " ")))
    {
      sscanf (pstr, "%s", str);

      if ((strcmp (str, name) == 0) || (strcmp (str, " ") == 0))
	{
	  /* nothing to do */

	}
      else if (strcmp (str, "hit") == 0)
	{
	  histo->conf.hit = TRUE;
	  if (debug)
	    fprintf (stderr, "hit,");

	}
      else if (strcmp (str, "avg") == 0)
	{
	  histo->conf.avg = TRUE;
	  if (debug)
	    fprintf (stderr, "avg,");

	}
      else if (strcmp (str, "min") == 0)
	{
	  histo->conf.min = TRUE;
	  if (debug)
	    fprintf (stderr, "min,");

	}
      else if (strcmp (str, "max") == 0)
	{
	  histo->conf.max = TRUE;
	  if (debug)
	    fprintf (stderr, "max,");

	}
      else if (strcmp (str, "var") == 0)
	{
	  histo->conf.var = TRUE;
	  if (debug)
	    fprintf (stderr, "var,");

	}
      else if (strcmp (str, "stdev") == 0)
	{
	  histo->conf.stdev = TRUE;
	  if (debug)
	    fprintf (stderr, "stdev,");

	}
      else if (strstr (str, "idx:"))
	{
	  sprintf (idxStr, "%s", str);
	}
      else if (strstr (str, "prc:"))
	{
	  sprintf (prcStr, "%s", str);
	}
      else
	{
	  fprintf (stderr,
		   "rrdtool: unknown token <%s> unknown while processing <%s> line %d\n",
		   str, rrd.conf.file, rrd.conf.line);
	  exit (1);
	}
    }

  if (strstr (idxStr, "idx:"))
    {
      char *substr = strtok (idxStr, ":,");	/* this gives ``idx'' ... */
      substr = strtok (NULL, ",");	/* ... so we skip it */

      while (substr != NULL)
	{
	  if (!strcmp (substr, "other"))
	    {
	      histo->conf.idxoth = TRUE;
	    }
	  else
	    {
	      ivec[iidx++] = atoi (substr);
	    }
	  substr = strtok (NULL, ",");
	}
      histo->conf.idxno = iidx;
      histo->conf.idx = (int *) malloc (sizeof (int) * histo->conf.idxno);
      if (debug)
	fprintf (stderr, "idx:");
      for (iidx = 0; iidx < histo->conf.idxno; iidx++)
	{
	  histo->conf.idx[iidx] = ivec[iidx];
	  if (debug)
	    fprintf (stderr, "%d:", ivec[iidx]);
	}
      if (debug && histo->conf.idxoth)
	fprintf (stderr, "other ");

    }

  if (strstr (prcStr, "prc:"))
    {
      char *substr = strtok (prcStr, ":,");	/* this gives ``prc'' ... */
      substr = strtok (NULL, ",");	/* ... so we skip it */

      while (substr != NULL)
	{
	  fvec[fidx++] = atof (substr);
	  substr = strtok (NULL, ",");
	}
      histo->conf.prcno = fidx;
      histo->conf.prc =
	(double *) malloc (sizeof (double) * histo->conf.prcno);
      if (debug)
	fprintf (stderr, "prc:");
      for (fidx = 0; fidx < histo->conf.prcno; fidx++)
	{
	  histo->conf.prc[fidx] = fvec[fidx];
	  if (debug)
	    fprintf (stderr, "%f:", fvec[fidx]);
	}
    }

  if (debug)
    fprintf (stderr, "\n");

}


/*============================================================================*
	tstat update/create frontend 
*-----------------------------------------------------------------------------*/

void
rrdtool_update_all ()
{
  struct double_histo_list *temphp;
  temphp = first_histo_list ();
  while (temphp != NULL)
    {
      rrdtool_update (temphp);
      temphp = temphp->next;
    }
}
void
rrdtool_create_all ()
{
  struct double_histo_list *temphp;
  temphp = first_histo_list ();
  while (temphp != NULL)
    {
      rrdtool_create (temphp);
      temphp = temphp->next;
    }
}

void
rrdtool_update (struct double_histo_list *h)
{
  struct rrdconf *conf = &h->conf;
  long *frozen = whats_frozen (h);
  int pos;
  int i, j, idx;
  int num;
  double prc, cdf;

  /* do not update more than once in the same instant */
  if (h->st.last.tv_sec >= current_time.tv_sec)
    return;
  h->st.last = current_time;

  if (!conf->rrd)
    return;
  if (conf->hit)
    rrdtool_update_command (h->name, "hit", h->st.count);
  if (conf->avg)
    rrdtool_update_command (h->name, "avg", h->st.avg);
  if (conf->min)
    rrdtool_update_command (h->name, "min", h->st.min);
  if (conf->max)
    rrdtool_update_command (h->name, "max", h->st.max);
  if (conf->var || conf->stdev)
    {
      double var = h->st.pseudovar / (h->st.count - 1);
      if (conf->var)
	rrdtool_update_command (h->name, "var", var);
      if (conf->stdev)
	rrdtool_update_command (h->name, "stdev", sqrt (var));
    }

  num = ((int) ((h->max - h->min) / h->bin_size)) + 1;

  if (conf->idxno)
    for (i = 0; i < conf->idxno; i++)
      {
	/* need to translate the index position in case bin_size != 1 */
	idx = conf->idx[i];
	pos = (idx - h->min) / h->bin_size + 1;
	sprintf (rrd.buf, "idx%d", idx);
	rrdtool_update_command (h->name, rrd.buf, frozen[pos]);
	/* recall than in histo[num+1] we count the hit number... */
	frozen[num + 1] -= frozen[pos];
      }
  if (conf->idxoth)
    rrdtool_update_command (h->name, "idxoth", frozen[num + 1]);

  if (conf->prcno)
    for (i = 0; i < conf->prcno; i++)
      {
	prc = conf->prc[i];
	if ((double) ((int) prc) == (double) prc)
	  {
	    sprintf (rrd.buf, "prc%.0f", conf->prc[i]);	/* prefer prc95 to prc95.000 */
	  }
	else
	  {
	    sprintf (rrd.buf, "prc%.3f", conf->prc[i]);	/* case prc99.999 */
	  }
	/* we do it the lazy way ... */
	prc = prc * frozen[num + 1] / 100;	/* get the hit number to stop at */
	cdf = 0.0;
	for (j = 0; j < num; j++)
	  {
	    if ((cdf += frozen[j]) >= prc)
	      {
		rrdtool_update_command (h->name, rrd.buf,
					h->min + (j - 1) * h->bin_size);
		break;
	      }
	  }
      }

  /* we now reset the stats as we reset the counters... 
     so that the min, var, max, etc stats are local to a specific
     time interval (otherwise min would be the minimum since rrdtool
     first started...) */
  h->st.count = 0;
  h->st.avg = 0.0;
  h->st.max = -1.e99;
  h->st.min = 1.e99;
  h->st.pseudovar = 0;
}


void
rrdtool_create (struct double_histo_list *h)
{
  struct rrdconf conf = h->conf;
  int i, idx;
  double prc;

  if (!conf.rrd)
    return;
  if (conf.avg)
    rrdtool_create_command (h->name, "avg");
  if (conf.min)
    rrdtool_create_command (h->name, "min");
  if (conf.max)
    rrdtool_create_command (h->name, "max");
  if (conf.var)
    rrdtool_create_command (h->name, "var");
  if (conf.idxoth)
    rrdtool_create_command (h->name, "idxoth");
  if (conf.stdev)
    rrdtool_create_command (h->name, "stdev");
  h->st.last.tv_sec = 0;	/* last update field */

  if (h->conf.idxno)
    for (i = 0; i < h->conf.idxno; i++)
      {
	idx = h->conf.idx[i];
	sprintf (rrd.buf, "idx%d", idx);
	rrdtool_create_command (h->name, rrd.buf);
      }
  if (h->conf.prcno)
    for (i = 0; i < h->conf.prcno; i++)
      {
	prc = h->conf.prc[i];
	if ((double) ((int) prc) == (double) prc)
	  {
	    sprintf (rrd.buf, "prc%.0f", prc);	/* prefer prc95 to prc95.000 */
	  }
	else
	  {
	    sprintf (rrd.buf, "prc%.3f", prc);	/* case prc99.999 */
	  }
	rrdtool_create_command (h->name, rrd.buf);
      }
}


/*============================================================================*
	rrd update/create library calls
*-----------------------------------------------------------------------------*/

void
rrdtool_create_command (const char *name, const char *what)
{
  int argc;
  char **argv;
  /* use a local copy to avoid overwriting the original rdd.cmd */
  rrd_struct temp_rrd = rrd;
  if (debug)
    fprintf (stderr, "rrdtool: create(%s,%s)\n", name, what);
#ifdef RRD_TREE
  sprintf (temp_rrd.file, "%s/%s", rrd.conf.path, name, what);
  struct stat fbuf;
  if (!((stat (temp_rrd.file, &fbuf) == 0) && S_ISDIR (fbuf.st_mode)))
    {
      mkdir (temp_rrd.file, 0775);
      if (debug > 1)
	fprintf (stderr, "RRDtool database path <%s> created\n",
		 temp_rrd.file);
    }
  sprintf (temp_rrd.file, "%s/%s/%s.rrd", rrd.conf.path, name, what);
#else
  sprintf (temp_rrd.file, "%s/%s.%s.rrd", rrd.conf.path, name, what);
#endif

  if (stat (temp_rrd.file, &temp_rrd.fbuf) == 0)
    {
      if (debug > 1)
	fprintf (stderr, "rrdtool: skip create <%s> ... already existent\n",
		 temp_rrd.file);
      return;			/* already called ? */
    }


  sprintf (temp_rrd.cmd,
	   "create %s --start %ld DS:%s:GAUGE:600:U:U %s %s %s %s",
	   temp_rrd.file, (long) current_time.tv_sec - 10, name, RRA_DAILY,
	   RRA_WEEKLY, RRA_MONTHLY, RRA_YEARLY);
  if (debug > 1)
    fprintf (stderr, "rrdtool: rrd_create('%s')\n", temp_rrd.cmd);

  optind = 0;
  opterr = 0;
  rrdtool_str2argv (temp_rrd.cmd, &argc, &argv);
  rrd_create (argc, argv);

  if (rrd_test_error ())
    {
      fprintf (stderr, "rrdtool: create command:\n%s\n", temp_rrd.cmd);
      fprintf (stderr, "rrdtool: create error!\n%s\n", rrd_get_error ());
      if (temp_rrd.fatal)
	exit (1);
      rrd_clear_error ();
    }
}

/*
 Syntax of rrdtool create 
   rrdtool create test.rrd       
            --start 920804400     
            DS:%d:LAST:600:U:U    
            RRA:AVERAGE:0.5:1:24   
            RRA:AVERAGE:0.5:6:10
	    
 as suggested in:
 http://people.ee.ethz.ch/~oetiker/webtools/rrdtool/tutorial/rrdtutorial.html	    
 
     1 sample "averaged" stays 1 period of 5 minutes
     6 samples averaged become one average on 30 minutes
     24 samples averaged become one average on 2 hours
     288 samples averaged become one average on 1 day

    Lets try to be compatible with MRTG: MRTG stores about the following amount of data:

     600 5-minute samples:    2   days and 2 hours
     600 30-minute samples:  12.5 days
     600 2-hour samples:     50   days
     732 1-day samples:     732   days

    These ranges are appended so the total amount of data kept is approximately 
    797 days. RRDtool stores the data differently, it doesn't start the ``weekly'' archive where the ``daily'' archive stopped. For both archives the most recent data will be near ``now'' and therefore we will need to keep more data than MRTG does!
    We will need:

     600 samples of 5 minutes  (2 days and 2 hours)
     700 samples of 30 minutes (2 days and 2 hours, plus 12.5 days)
     775 samples of 2 hours    (above + 50 days)
     797 samples of 1 day      (above + 732 days, rounded up to 797)	    
*/



void
rrdtool_update_command (const char *name, const char *what, double val)
{
  int argc;
  char **argv;
  if (debug)
    fprintf (stderr, "rrdtool: update(%s,%s,%f)\n", name, what, val);
#ifdef RRD_TREE
  sprintf (rrd.file, "%s/%s/%s.rrd", rrd.conf.path, name, what);
#else
  sprintf (rrd.file, "%s/%s.%s.rrd", rrd.conf.path, name, what);
#endif

  /* at the first call of this function, all the needed rrd 
     files are created with the start time set appropriately */
  if (stat (rrd.file, &rrd.fbuf) == -1)
    rrdtool_create_all ();

  sprintf (rrd.cmd, "update %s %ld:%f",
	   rrd.file, (long) current_time.tv_sec, val);
  if (debug)
    fprintf (stderr, "rrdtool: rrd_update('%s')\n", rrd.cmd);

  optind = 0;
  opterr = 0;
  rrdtool_str2argv (rrd.cmd, &argc, &argv);
  rrd_update (argc, argv);

  if (rrd_test_error ())
    {
      fprintf (stderr, "rrdtool: update command:\n%s\n", rrd.cmd);
      fprintf (stderr, "rrdtool: update error!\n%s\n", rrd_get_error ());
      if (rrd.fatal)
	exit (1);
      rrd_clear_error ();
    }
}





struct double_histo_list *
find_histo (char *name)
{
  struct double_histo_list *histo = first_histo_list ();

  while (histo)
    {
      if (!strcmp (histo->name, name))
	return histo;
      histo = histo->next;
    }
  return NULL;
}

/*-----------------------------------------------------------*/
#endif
