/*
 *
 * Copyright (c) 2001
 *	Politecnico di Torino.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Author:	Marco Mellia, Andrea Carpani, Luca Muscariello, Dario Rossi
 * 		Telecomunication Networks Group
 * 		Politecnico di Torino
 * 		Torino, Italy
 *              http://http://www.tlc-networks.polito.it/index.html
 *		mellia@mail.tlc.polito.it
 *
 * Tstat is deeply based on TCPTRACE. The authors would like to thank
 * Shawn Ostermann for the development of TCPTRACE.
 *
*/

#include<stdio.h>
#include "tstat.h"

extern unsigned long int f_RTP_count;
extern unsigned long int f_RTCP_count;
extern FILE *fp_rtp_logc;
#ifdef HAVE_RRDTOOL
/*----------------------------------------------------------*/
/*   RRDtools 				                    */
extern Bool pure_rrd_engine;
#endif

/* variables used to compute the RTP packets fields*/

u_int32_t pssrc;
u_int32_t pts;
u_int16_t pseq;

/******** function used to find the RTP packet starting point **********/

struct rtphdr *
getrtp (struct udphdr *pudp, void *plast)
{

  struct rtphdr *prtp;
  void *theheader;

  theheader = ((char *) pudp + 8);
  if ((u_long) theheader + (sizeof (struct rtphdr)) - 1 > (u_long) plast)
    {
      /* part of the header is missing */
      return (NULL);
    }
  prtp = (struct rtphdr *) theheader;
  return (prtp);
}



/***************** function used to manage the flows *****************/


void
rtpdotrace (ucb * thisdir, struct rtphdr *prtp, int dir, struct ip *pip)
{
  pssrc = swap32 (prtp->ssrc);
  pts = swap32 (prtp->ts);
  pseq = swap16 (prtp->seqno);

  switch (thisdir->type)
    {
    case unknown:
      {
	init_rtp (thisdir, prtp);
	break;
      }
    case first_RTP:
      {
	/* already got a packet ... double check it */
	rtp_check (thisdir, prtp, dir, pip);
	break;
      }
    case first_RTCP:
      {
	/* already got a packet ... double check it */
	rtcp_check (thisdir, prtp);
	break;
      }
    case RTP:
      {
	/* already identified the RTP flow... */
	struct rtp *f_rtp;
	f_rtp = &thisdir->flow.rtp;

	if ((prtp->v == VALID_VERSION) && (f_rtp->ssrc == pssrc)
	    && (prtp->pt < VALID_PT))
	  rtp_stat (thisdir, f_rtp, prtp, dir, pip);
	else
	  {
	    /* The RTP flow is closed but not the UDP one */
	    /* this should not happen... */
	    make_rtp_conn_stats (thisdir, dir);
	    thisdir->type = unknown;
	  }
	break;
      }

    case RTCP:
      {
	/* already identified the RTCP flow... */
	struct rtcp *f_rtcp;
	struct sudp_pair *pup;
	f_rtcp = &thisdir->flow.rtcp;
	pup = thisdir->pup;

	if ((prtp->v == VALID_VERSION) &&
	    ((prtp->pt + (prtp->m << 7)) >= RTCP_MIN_PT) &&
	    ((prtp->pt + (prtp->m << 7)) <= RTCP_MAX_PT) &&
	    ((pup->addr_pair.a_port & 1) == 1) &&
	    (pup->addr_pair.a_port > 1024))
	  rtcp_stat (f_rtcp, prtp);
	else
	  {
	    /* The RTCP flow is closed but not the UDP one */
	    make_rtcp_conn_stats (thisdir, dir);
	    thisdir->type = unknown;
	  }
	break;
      }
    }
}





/**** function used to initialize the parameters of the flows ****/

void
init_rtp (ucb * thisdir, struct rtphdr *prtp)
{
  struct rtp *f_rtp;
  struct rtcp *f_rtcp;
  struct sudp_pair *pup;
  int i;

  pup = thisdir->pup;
  /* this may be RTP or RTCP...
     version must be 2
     pt must be RTP valid
     udp port must be odd and greater than 1024
     then it is a RTCP flow... otherwise it may bw a RTP flow
   */
  if ((prtp->v == VALID_VERSION) &&
      ((prtp->pt + (prtp->m << 7)) >= RTCP_MIN_PT) &&
      ((prtp->pt + (prtp->m << 7)) <= RTCP_MAX_PT) &&
      ((pup->addr_pair.a_port & 1) == 1) && (pup->addr_pair.a_port > 1024) &&
      ((pup->addr_pair.b_port & 1) == 1) && (pup->addr_pair.b_port > 1024))
    {

      f_rtcp = &thisdir->flow.rtcp;
      f_rtcp->pnum = 1;
      f_rtcp->sum_delta_t = 0;
      f_rtcp->first_time = current_time;
      f_rtcp->last_time = current_time;
      thisdir->type = first_RTCP;
    }
  else if ((prtp->v == VALID_VERSION) &&
	   (prtp->pt < VALID_PT) &&
	   ((pup->addr_pair.a_port & 1) == 0) &&
	   (pup->addr_pair.a_port > 1024))
    {
      f_rtp = &thisdir->flow.rtp;

      f_rtp->initial_seqno = pseq;
      f_rtp->largest_seqno = pseq;
      f_rtp->pnum = 1;		/* is the first packet of the RTP flow */
      f_rtp->ssrc = pssrc;
      f_rtp->first_time = current_time;
      f_rtp->last_time = current_time;
      f_rtp->n_out_of_sequence = 0;
      f_rtp->sum_delta_t = 0;
      f_rtp->n_delta_t = 0;
      f_rtp->transit = 0;
      f_rtp->first_ts = pts;
      f_rtp->largest_ts = pts;
      f_rtp->jitter = 0;
      f_rtp->n_dup = 0;
      f_rtp->n_late = 0;
      f_rtp->n_lost = 0;
      f_rtp->burst = 0;

      f_rtp->w = FALSE;
      for (i = 0; i < WIN; i++)	/* reset of the window at the beginning of the flow */
	f_rtp->packets_win[i] = pseq;
      thisdir->type = first_RTP;
    }


}


/* function uesd to control if the current analized packet is an RTP packet */

void
rtp_check (ucb * thisdir, struct rtphdr *prtp, int dir, struct ip *pip)
{
  struct rtp *f_rtp = &thisdir->flow.rtp;

  if ((prtp->v == VALID_VERSION) && (f_rtp->ssrc == pssrc) &&
      (f_rtp->initial_seqno + (u_int16_t) f_rtp->pnum == pseq) &&
      (prtp->pt < VALID_PT))
    {
      f_RTP_count++;
      rtp_stat (thisdir, f_rtp, prtp, dir, pip);
      thisdir->type = RTP;
    }
  else				/* is not an RTP packet */
    thisdir->type = unknown;
}

/* function used to control if the current analized packet is an RTCP packet*/

void
rtcp_check (ucb * thisdir, struct rtphdr *prtp)
{
  struct rtcp *f_rtcp = &thisdir->flow.rtcp;

  if ((prtp->v == VALID_VERSION) &&
      ((prtp->pt + (prtp->m << 7)) >= RTCP_MIN_PT) &&
      ((prtp->pt + (prtp->m << 7)) <= RTCP_MAX_PT) && (f_rtcp->pnum == 1))
    {
      f_RTCP_count++;
      rtcp_stat (f_rtcp, prtp);
      thisdir->type = RTCP;
    }
  else				/* is not an RTCP packet */
    thisdir->type = unknown;
}

/******* Statistichs about RTP flows ********/

void
rtp_stat (ucb * thisdir, struct rtp *f_rtp, struct rtphdr *prtp, int dir,
	  struct ip *pip)
{
  struct sudp_pair *pup;
  int i, freq, index, delta_seq, delta_seq_win;
  double period;
  double delta_t, t, ts, transit, d;

  pup = thisdir->pup;
  f_rtp->pnum++;

  freq = det_freq (prtp);
  if (freq != 0)
    period = 1000. / freq;
  else
    period = 1000.;


  /* delta_t evaluation if packets are in sequence */
  if ((u_int16_t) (f_rtp->largest_seqno + 1) == pseq)
    {
      /* delta_t in milliseconds */
      delta_t = elapsed (f_rtp->last_time, current_time);
      delta_t = delta_t / 1000;
      f_rtp->sum_delta_t += delta_t;
      f_rtp->n_delta_t++;


      /* jitter evaluation in milliseconds */

      if (freq != 0)
	{
	  /* alignment of the scales */

	  t = elapsed (f_rtp->first_time, current_time);
	  t = t / 1000.0;
	  ts = ((pts - f_rtp->first_ts) * period);

	  /* jitter is computed as in the definition from RFC 3550 */
	  transit = t - ts;
	  d = transit - f_rtp->transit;
	  f_rtp->transit = transit;
	  if (d < 0)
	    d = -d;
	  f_rtp->jitter += (1. / 16.) * (d - f_rtp->jitter);
	}

      /* end jitter evaluation */
    }

/** management of the window used for oos, duplicate, late or lost packets **/

  if ((u_int16_t) (pseq - f_rtp->initial_seqno) >= WIN)
    f_rtp->w = TRUE;		/* got data for at least 1 window */

  index = ((u_int16_t) (pseq - f_rtp->initial_seqno) % WIN);
  delta_seq_win = (u_int16_t) (pseq - f_rtp->packets_win[index]);

  /* if there is a sudden jump in the sequence number 
     it happens that some bogus implementation of rtp Oencoders 
     change the sequence numbering at random (CISCO...)
     So try to detect them */
  if (delta_seq_win > WIN * 5
      && (u_int32_t) (pts - f_rtp->largest_ts) < 0xffff0000
      && f_rtp->w == TRUE)
    {
      f_rtp->w = FALSE;		/* window not valid */
      f_rtp->largest_seqno = pseq;
      f_rtp->initial_seqno = pseq - 1;
      for (i = 0; i < WIN; i++)	/* reset of the window at the beginning of the flow */
	f_rtp->packets_win[i] = pseq - 1;

      index = ((u_int16_t) (pseq - f_rtp->initial_seqno) % WIN);	/* equal to 1 */
      delta_seq_win = (u_int16_t) (pseq - f_rtp->packets_win[index]);	/* idem */
    }

  /* look at the largest ts */
  if ((u_int32_t) (pts - f_rtp->largest_ts) < OVERFLOW_TH)
    /* control of the overflow!! */
    f_rtp->largest_ts = pts;

  if (f_rtp->w == FALSE)	/* first time using the window */
    {

      if (delta_seq_win == 0)
	{
	  f_rtp->n_dup++;
	}

      f_rtp->packets_win[index] = pseq;

      /* if the sequence number is not equal to largest_seqno+1 is 
         considered an out of sequence */
      if ((u_int16_t) (f_rtp->largest_seqno + 1) != pseq)
	{
	  f_rtp->n_out_of_sequence++;

	  if (pup->internal_src && !pup->internal_dst)
	    {
	      if (dir == C2S)
		add_histo (rtp_oos_p_out, 1);
	      else
		add_histo (rtp_oos_p_in, 1);

	    }
	  else if (!pup->internal_src && pup->internal_dst)
	    {
	      if (dir == C2S)
		add_histo (rtp_oos_p_in, 1);

	      else
		add_histo (rtp_oos_p_out, 1);

	    }
	  else if (pup->internal_src && pup->internal_dst)
	    {
	      add_histo (rtp_oos_p_loc, 1);
	    }

	  if (((u_int16_t) (pseq - f_rtp->largest_seqno) > OVERFLOW_TH) &&
	      (delta_seq_win != 0))
	    {
	      double delay;
	      float byte_period;

	      /* evaluate the delay of the oos segment */
	      byte_period = (ntohs (pip->ip_len) - ntohs (pip->ip_hl) - 8	/* udp header len */
			     - (12 + ntohs (prtp->cc) * 4)	/* rtp header len */
		) * period;
	      delay =
		((u_int16_t) (f_rtp->largest_seqno - pseq) * byte_period) +
		(((double) elapsed (f_rtp->last_time, current_time)) /
		 1000.0);
	      if (pup->internal_src && !pup->internal_dst)
		{
		  if (dir == C2S)
		    {
		      add_histo (rtp_reord_p_n_out, 1);
		      add_histo (rtp_reord_delay_out, delay);
		    }
		  else
		    {
		      add_histo (rtp_reord_p_n_in, 1);
		      add_histo (rtp_reord_delay_in, delay);
		    }
		}
	      else if (!pup->internal_src && pup->internal_dst)
		{
		  if (dir == C2S)
		    {
		      add_histo (rtp_reord_p_n_in, 1);
		      add_histo (rtp_reord_delay_in, delay);
		    }
		  else
		    {
		      add_histo (rtp_reord_p_n_out, 1);
		      add_histo (rtp_reord_delay_out, delay);
		    }
		}
	      else if (pup->internal_src && pup->internal_dst)
		{
		  add_histo (rtp_reord_p_n_loc, 1);
		  add_histo (rtp_reord_delay_loc, delay);
		}
	    }
	}

      if ((u_int16_t) (pseq - f_rtp->largest_seqno) < OVERFLOW_TH)
	/* control of the overflow!! */
	f_rtp->largest_seqno = pseq;
    }
  else				/* got the window data filled up */
    {
      delta_seq = (u_int16_t) (pseq - f_rtp->largest_seqno);

      /* This packet is late than it doesn't belong to the present window */
      if (delta_seq_win >= OVERFLOW_TH)
	{
	  f_rtp->n_late++;
	}
      else if (delta_seq_win == 0)
	{
	  f_rtp->n_dup++;
	}
      else if (delta_seq_win > WIN)
	{
	  f_rtp->n_lost++;
	  f_rtp->burst++;
	  f_rtp->packets_win[index] = pseq;
	}
      else if (delta_seq_win == WIN)
	{
	  /* in sequence */
	  /* Statistics of the burst and reset of the burst length */
	  if (pup->internal_src && !pup->internal_dst)
	    {
	      if (dir == C2S)
		add_histo (rtp_burst_loss_out, f_rtp->burst);
	      else
		add_histo (rtp_burst_loss_in, f_rtp->burst);
	    }
	  else if (!pup->internal_src && pup->internal_dst)
	    {
	      if (dir == C2S)
		add_histo (rtp_burst_loss_in, f_rtp->burst);
	      else
		add_histo (rtp_burst_loss_out, f_rtp->burst);
	    }
	  else if (pup->internal_src && pup->internal_dst)
	    {
	      add_histo (rtp_burst_loss_loc, f_rtp->burst);
	    }

	  f_rtp->burst = 0;
	  f_rtp->packets_win[index] = pseq;
	}

      /* if the sequence number is not equal to largest_seqno+1 is considered an
         out of sequence */
      if ((u_int16_t) (f_rtp->largest_seqno + 1) != pseq)
	{
	  f_rtp->n_out_of_sequence++;

	  if (pup->internal_src && !pup->internal_dst)
	    {
	      if (dir == C2S)
		add_histo (rtp_oos_p_out, 1);
	      else
		add_histo (rtp_oos_p_in, 1);

	    }
	  else if (!pup->internal_src && pup->internal_dst)
	    {
	      if (dir == C2S)
		add_histo (rtp_oos_p_in, 1);

	      else
		add_histo (rtp_oos_p_out, 1);

	    }
	  else if (pup->internal_src && pup->internal_dst)
	    {
	      add_histo (rtp_oos_p_loc, 1);
	    }

	  if (((u_int16_t) (pseq - f_rtp->largest_seqno) > OVERFLOW_TH) &&
	      (delta_seq_win != 0))
	    {

	      double delay;
	      float byte_period;

	      byte_period = (ntohs (pip->ip_len) - ntohs (pip->ip_hl) - 8	/* udp header len */
			     - (12 + ntohs (prtp->cc) * 4)	/* rtp header len */
		) * period;
	      delay =
		((u_int16_t) (f_rtp->largest_seqno - pseq) * byte_period) +
		(((double) elapsed (f_rtp->last_time, current_time)) /
		 1000.0);
	      if (pup->internal_src && !pup->internal_dst)
		{
		  if (dir == C2S)
		    {
		      add_histo (rtp_reord_p_n_out, 1);
		      add_histo (rtp_reord_delay_out, delay);
		    }
		  else
		    {
		      add_histo (rtp_reord_p_n_in, 1);
		      add_histo (rtp_reord_delay_in, delay);
		    }
		}
	      else if (!pup->internal_src && pup->internal_dst)
		{
		  if (dir == C2S)
		    {
		      add_histo (rtp_reord_p_n_in, 1);
		      add_histo (rtp_reord_delay_in, delay);
		    }
		  else
		    {
		      add_histo (rtp_reord_p_n_out, 1);
		      add_histo (rtp_reord_delay_out, delay);
		    }
		}
	      else if (pup->internal_src && pup->internal_dst)
		{
		  add_histo (rtp_reord_p_n_loc, 1);
		  add_histo (rtp_reord_delay_loc, delay);
		}
	    }
	}

      if ((u_int16_t) (pseq - f_rtp->largest_seqno) < OVERFLOW_TH)
	/* control of the overflow!! */
	f_rtp->largest_seqno = pseq;
    }

  f_rtp->last_time = current_time;
}



/******* Statistics about RTCP flows ********/

void
rtcp_stat (struct rtcp *f_rtcp, struct rtphdr *prtp)
{
  double delta_t;

  f_rtcp->pnum++;		/* rtcp packets number */
  /* delta_t calculus in  milliseconds */
  delta_t = elapsed (f_rtcp->last_time, current_time);
  delta_t = delta_t / 1000;
  f_rtcp->sum_delta_t += delta_t;

  f_rtcp->last_time = current_time;
}



/******** function to update the RTP histograms *********/

void
make_rtp_conn_stats (ucb * thisdir, int dir)
{
  struct sudp_pair *pup;
  struct rtp *f_rtp;
  FILE *fprtp;
  char logline[300] = "";
  double etime;
  int max_index, index, min_val;

  fprtp = fp_rtp_logc;
  f_rtp = &thisdir->flow.rtp;
  pup = thisdir->pup;

  /* Calculus of the RTP flow length in seconds */

  etime = elapsed (f_rtp->first_time, f_rtp->last_time);
  etime = etime / 1000000;


/* analysis of the last window before the closing of the flow */
/* maximum index reached in the window */
  max_index =
    ((u_int16_t) (f_rtp->largest_seqno - f_rtp->initial_seqno) % WIN);
/* value that must be contained in the first elemet of the window */
  min_val = (u_int16_t) (f_rtp->largest_seqno - max_index);
  for (index = 0; index <= max_index; index++)
    {
      if (f_rtp->packets_win[index] != (u_int16_t) (min_val + index))
	{
	  f_rtp->n_lost++;
	  f_rtp->burst++;
	}
      else
	{
	  if (pup->internal_src && !pup->internal_dst)
	    {
	      if (dir == C2S)
		add_histo (rtp_burst_loss_out, f_rtp->burst);
	      else
		add_histo (rtp_burst_loss_in, f_rtp->burst);
	    }
	  else if (!pup->internal_src && pup->internal_dst)
	    {
	      if (dir == C2S)
		add_histo (rtp_burst_loss_in, f_rtp->burst);
	      else
		add_histo (rtp_burst_loss_out, f_rtp->burst);
	    }
	  else if (pup->internal_src && pup->internal_dst)
	    {
	      add_histo (rtp_burst_loss_loc, f_rtp->burst);
	    }
	  f_rtp->burst = 0;
	}
    }

  add_histo (flow_number, HISTO_FLOW_RTP);

  if (pup->internal_src && !pup->internal_dst)
    {

      if (dir == C2S)
	{
	  add_histo (rtp_cl_p_out, f_rtp->pnum);
	  add_histo (rtp_avg_delay_out,
		     10. * (f_rtp->sum_delta_t / f_rtp->n_delta_t));
	  add_histo (rtp_avg_jitter_out, 10. * (f_rtp->jitter));
	  add_histo (rtp_n_oos_out, f_rtp->n_out_of_sequence);
	  add_histo (rtp_p_oos_out,
		     ((float) f_rtp->n_out_of_sequence /
		      (float) f_rtp->pnum) * 1000);
	  add_histo (rtp_tot_time_out, etime);
	  add_histo (rtp_p_dup_out, (f_rtp->n_dup * 1000) / f_rtp->pnum);
	  add_histo (rtp_p_lost_out, (f_rtp->n_lost * 1000) / f_rtp->pnum);
	  add_histo (rtp_p_late_out, (f_rtp->n_late * 1000) / f_rtp->pnum);
	}
      else
	{
	  add_histo (rtp_cl_p_in, f_rtp->pnum);
	  add_histo (rtp_avg_delay_in,
		     10. * (f_rtp->sum_delta_t / f_rtp->n_delta_t));
	  add_histo (rtp_avg_jitter_in, 10. * (f_rtp->jitter));
	  add_histo (rtp_n_oos_in, f_rtp->n_out_of_sequence);
	  add_histo (rtp_p_oos_in,
		     ((float) f_rtp->n_out_of_sequence /
		      (float) f_rtp->pnum) * 1000);
	  add_histo (rtp_tot_time_in, etime);
	  add_histo (rtp_p_dup_in, (f_rtp->n_dup * 1000) / f_rtp->pnum);
	  add_histo (rtp_p_lost_in, (f_rtp->n_lost * 1000) / f_rtp->pnum);
	  add_histo (rtp_p_late_in, (f_rtp->n_late * 1000) / f_rtp->pnum);
	}
    }
  else if (!pup->internal_src && pup->internal_dst)
    {
      if (dir == C2S)
	{
	  add_histo (rtp_cl_p_in, f_rtp->pnum);
	  add_histo (rtp_avg_delay_in,
		     10. * (f_rtp->sum_delta_t / f_rtp->n_delta_t));
	  add_histo (rtp_avg_jitter_in, 10. * (f_rtp->jitter));
	  add_histo (rtp_n_oos_in, f_rtp->n_out_of_sequence);
	  add_histo (rtp_p_oos_in,
		     ((float) f_rtp->n_out_of_sequence /
		      (float) f_rtp->pnum) * 1000);
	  add_histo (rtp_tot_time_in, etime);
	  add_histo (rtp_p_dup_in, (f_rtp->n_dup * 1000) / f_rtp->pnum);
	  add_histo (rtp_p_lost_in, (f_rtp->n_lost * 1000) / f_rtp->pnum);
	  add_histo (rtp_p_late_in, (f_rtp->n_late * 1000) / f_rtp->pnum);
	}
      else
	{
	  add_histo (rtp_cl_p_out, f_rtp->pnum);
	  add_histo (rtp_avg_delay_out,
		     10. * (f_rtp->sum_delta_t / f_rtp->n_delta_t));
	  add_histo (rtp_avg_jitter_out, 10. * (f_rtp->jitter));
	  add_histo (rtp_n_oos_out, f_rtp->n_out_of_sequence);
	  add_histo (rtp_p_oos_out,
		     ((float) f_rtp->n_out_of_sequence /
		      (float) f_rtp->pnum) * 1000);
	  add_histo (rtp_tot_time_out, etime);
	  add_histo (rtp_p_dup_out, (f_rtp->n_dup * 1000) / f_rtp->pnum);
	  add_histo (rtp_p_lost_out, (f_rtp->n_lost * 1000) / f_rtp->pnum);
	  add_histo (rtp_p_late_out, (f_rtp->n_late * 1000) / f_rtp->pnum);

	}
    }
  else if (pup->internal_src && pup->internal_dst)
    {
      add_histo (rtp_cl_p_loc, f_rtp->pnum);
      add_histo (rtp_avg_delay_loc,
		 10. * (f_rtp->sum_delta_t / f_rtp->n_delta_t));
      add_histo (rtp_avg_jitter_loc, 10. * (f_rtp->jitter));
      add_histo (rtp_n_oos_loc, f_rtp->n_out_of_sequence);
      add_histo (rtp_p_oos_loc,
		 ((float) f_rtp->n_out_of_sequence / (float) f_rtp->pnum) *
		 1000);
      add_histo (rtp_tot_time_loc, etime);
      add_histo (rtp_p_dup_loc, (f_rtp->n_dup * 1000) / f_rtp->pnum);
      add_histo (rtp_p_lost_loc, (f_rtp->n_lost * 1000) / f_rtp->pnum);
      add_histo (rtp_p_late_loc, (f_rtp->n_late * 1000) / f_rtp->pnum);
    }

#ifdef HAVE_RRDTOOL
  if (pure_rrd_engine)
    return;
#endif

  if (dir == C2S)
    {
      sprintf (logline, "%s %s %lu %u %g %g  %ld %ld %ld %ld",
	       HostName (pup->addr_pair.a_address),
	       ServiceName (pup->addr_pair.a_port),
	       f_rtp->pnum,
	       f_rtp->ssrc,
	       (f_rtp->sum_delta_t / f_rtp->n_delta_t),
	       f_rtp->jitter,
	       f_rtp->n_out_of_sequence,
	       f_rtp->n_dup, f_rtp->n_late, f_rtp->n_lost);

      sprintf (logline, "%s %d %d", logline, pup->internal_src,
	       pup->internal_dst);

      /* scrivo nel file f_rtp_logc */
      fprintf (fprtp, "%s\n", logline);
    }
  else
    {
      sprintf (logline, "%s %s %lu %u %g %g  %ld %ld %ld %ld",
	       HostName (pup->addr_pair.b_address),
	       ServiceName (pup->addr_pair.b_port),
	       f_rtp->pnum,
	       f_rtp->ssrc,
	       (f_rtp->sum_delta_t / f_rtp->n_delta_t),
	       f_rtp->jitter,
	       f_rtp->n_out_of_sequence,
	       f_rtp->n_dup, f_rtp->n_late, f_rtp->n_lost);

      sprintf (logline, "%s %d %d", logline, pup->internal_src,
	       pup->internal_dst);

      /* scrivo nel file f_rtp_logc */
      fprintf (fprtp, "%s\n", logline);
    }
}



/******** function used to update the RTCP histograms *********/

void
make_rtcp_conn_stats (ucb * thisdir, int dir)
{
  struct sudp_pair *pup;
  struct rtcp *f_rtcp;

  f_rtcp = &thisdir->flow.rtcp;
  pup = thisdir->pup;
  add_histo (flow_number, HISTO_FLOW_RTCP);

  if (pup->internal_src && !pup->internal_dst)
    {

      if (dir == C2S)
	{
	  add_histo (rtcp_cl_p_out, f_rtcp->pnum);
	  add_histo (rtcp_avg_inter_out,
		     (float) f_rtcp->sum_delta_t / (float) f_rtcp->pnum);
	}
      else
	{
	  add_histo (rtcp_cl_p_in, f_rtcp->pnum);
	  add_histo (rtcp_avg_inter_in,
		     (float) f_rtcp->sum_delta_t / (float) f_rtcp->pnum);
	}
    }
  else if (!pup->internal_src && pup->internal_dst)
    {
      if (dir == C2S)
	{
	  add_histo (rtcp_cl_p_in, f_rtcp->pnum);
	  add_histo (rtcp_avg_inter_in,
		     (float) f_rtcp->sum_delta_t / (float) f_rtcp->pnum);

	}
      else
	{
	  add_histo (rtcp_cl_p_out, f_rtcp->pnum);
	  add_histo (rtcp_avg_inter_out,
		     (float) f_rtcp->sum_delta_t / (float) f_rtcp->pnum);

	}
    }
  else if (pup->internal_src && pup->internal_dst)
    {
      add_histo (rtcp_cl_p_loc, f_rtcp->pnum);
      add_histo (rtcp_avg_inter_loc,
		 (float) f_rtcp->sum_delta_t / (float) f_rtcp->pnum);
    }

}



/* function used to interprete the 16 bit values whether in the BIG ENDIAN case
or in the  LITTLE ENDIAN one */

u_int16_t
swap16 (u_int16_t val)
{
#ifdef RTP_LITTLE_ENDIAN
  return ((val & 0xff00) >> 8) | ((val & 0x00ff) << 8);
#else
  return val;
#endif
}



/* function used to interprete the 32 bit values whether in the BIG ENDIAN case
or in the  LITTLE ENDIAN one */

u_int32_t
swap32 (u_int32_t val)
{
#ifdef RTP_LITTLE_ENDIAN
  return
    ((val & 0xff000000) >> 24) |
    ((val & 0x00ff0000) >> 8) |
    ((val & 0x0000ff00) << 8) | ((val & 0x000000ff) << 24);
#else
  return val;
#endif
}



/** function used to determine the sampling frequence indicated by the 
'Payload Type' field **/

int
det_freq (struct rtphdr *prtp)
{
  switch (prtp->pt)
    {
    case RTP_PT_PCMU:
    case RTP_PT_CELP:
    case RTP_PT_G711:
    case RTP_PT_DVI4_8000:
    case RTP_PT_GSM:
    case RTP_PT_LPC:
    case RTP_PT_PCMA:
    case RTP_PT_G722:
    case RTP_PT_G728:
      {
	return (8000);
	break;
      }
    case RTP_PT_L16_2:
    case RTP_PT_L16_1:
      {
	return (44100);
	break;
      }
    case RTP_PT_DVI4_16000:
      {
	return (16000);
	break;
      }

    case RTP_PT_MPA:
    case RTP_PT_CELLB:
    case RTP_PT_JPEG:
    case RTP_PT_NV:
    case RTP_PT_H261:
    case RTP_PT_MPEG:
    case RTP_PT_MP2T:
      {
	return (90000);
	break;
      }
    default:
      {
	return (0);
	break;
      }
    }
}
