/*
 *
 * Copyright (c) 2001
 *	Politecnico di Torino.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Author:	Marco Mellia, Andrea Carpani, Luca Muscariello, Dario Rossi
 * 		Telecomunication Networks Group
 * 		Politecnico di Torino
 * 		Torino, Italy
 *              http://http://www.tlc-networks.polito.it/index.html
 *		mellia@mail.tlc.polito.it
 *
 * Tstat is deeply based on TCPTRACE. The authors would like to thank
 * Shawn Ostermann for the development of TCPTRACE.
 *
*/


/* we want LONG LONG in some places */
#if SIZEOF_UNSIGNED_LONG_LONG_INT >= 8
#define HAVE_LONG_LONG
typedef unsigned long long int u_llong;
typedef long long int llong;
#else /* LONG LONG */
typedef unsigned long int u_llong;
typedef long int llong;
#endif /* LONG LONG */

/* type for a TCP sequence number, ACK, FIN, or SYN */
typedef u_long seqnum;

/* length of a segment */
typedef u_long seglen;

/* type for a quadrant number */
typedef u_char quadnum;		/* 1,2,3,4 */

/* type for a TCP port number */
typedef u_short portnum;

/* type for an IP address */
/* IP address can be either IPv4 or IPv6 */
typedef struct ipaddr
{
  u_char addr_vers;		/* 4 or 6 */
  union
  {
    struct in_addr ip4;
#ifdef SUPPORT_IPV6
    struct in6_addr ip6;
#endif
  }
  un;
}
ipaddr;


/* type for a timestamp */
typedef struct timeval timeval;
#define ZERO_TIME(ptv)(((ptv)->tv_sec == 0) && ((ptv)->tv_usec == 0))
#define time2double(t) ((double) (t).tv_sec * 1000000 + (double) (t).tv_usec)

/* type for a Boolean */
typedef u_char Bool;
#define TRUE	1
#define FALSE	0
#define BOOL2STR(b) (b)?"TRUE":"FALSE"

/* ACK types */
enum t_ack
{ NORMAL = 1,			/* no retransmits, just advance */
  AMBIG = 2,			/* segment ACKed was rexmitted */
  CUMUL = 3,			/* doesn't advance */
  TRIPLE = 4,			/* triple dupack */
  NOSAMP = 5
};				/* covers retransmitted segs, no rtt sample */

/* type for an internal file pointer */
typedef struct mfile MFILE;

extern unsigned long int fcount;	/* total flow number number */

/* first and last packet timestamp */
extern timeval first_packet;
extern timeval last_packet;

/* counters */
// LM
extern u_long tcp_trace_count_outgoing;
extern u_long tcp_trace_count_incoming;
extern u_long tcp_trace_count_local;
extern u_long udp_trace_count;


#ifdef OLD
/* test 2 IP addresses for equality */
#define IP_SAMEADDR(addr1,addr2) (((addr1).s_addr) == ((addr2).s_addr))

/* copy IP addresses */
#define IP_COPYADDR(toaddr,fromaddr) ((toaddr).s_addr = (fromaddr).s_addr)
#endif

typedef struct segment
{
  seqnum seq_firstbyte;		/* seqnumber of first byte */
  seqnum seq_lastbyte;		/* seqnumber of last byte */
  u_char retrans;		/* retransmit count */
  u_int acked;			/* how MANY times has has it been acked? */
  timeval time;			/* time the segment was sent */
  /* LM start - add field to implement an heuristic to identify 
     loss packets within a flow
   */
  u_short ip_id;		/* 16 bit ip identification field  */
  char type_of_segment;
  /* LM stop */
  struct segment *next;
  struct segment *prev;
}
segment;

typedef struct quadrant
{
  segment *seglist_head;
  segment *seglist_tail;
  Bool full;
  u_long no_of_segments;
  struct quadrant *prev;
  struct quadrant *next;
}
quadrant;

typedef struct seqspace
{
  quadrant *pquad[4];
}
seqspace;

typedef struct tcb
{
  /* parent pointer */
  struct stcp_pair *ptp;
  struct tcb *ptwin;

  seqnum highest_seqno;		/* to identify the end of the data */
  Bool data_to_be_acked;	/* true if waiting for an ack */
  timeval payload_start_time;	/* time of first data pkt ... */
  timeval payload_end_time;	/* time of last valid ack... */

  /* TCP information */
  seqnum ack;
  seqnum seq;
  seqnum syn;
  seqnum fin;
  seqnum fin_seqno;
  seqnum windowend;
  timeval time;

  /* TCP options */
  u_int mss;
  Bool f1323_ws;		/* did he request 1323 window scaling? */
  Bool f1323_ts;		/* did he request 1323 timestamps? */
  Bool fsack_req;		/* did he request SACKs? */
  u_char window_scale;

  /* statistics added */
  u_long data_bytes;
  u_long data_pkts;
  u_long data_pkts_push;
  u_long unique_bytes;		/* bytes sent (-FIN/SYN), excluding rexmits */
  u_int rexmit_bytes;
  u_int rexmit_pkts;
  u_long ack_pkts;
  u_long pureack_pkts;		/* mallman - pure acks, no data */
  u_int win_max;
  u_int win_min;
  u_int win_tot;
  u_int win_curr;
  u_int win_zero_ct;
  u_int min_seq;
  u_int max_seq;
  u_long packets;
  u_char syn_count;
  u_char fin_count;
  Bool closed;
  u_char reset_count;		/* resets SENT */
  u_int min_seg_size;
  u_int max_seg_size;
  u_int out_order_pkts;		/* out of order packets */
  u_int sacks_sent;		/* sacks returned */

  /* .a.c. cwnd stats */
  u_int cwnd_max;
  u_int cwnd_flag;

  /* did I detect any "bad" tcp behavior? */
  /* at present, this means: */
  /*  - SYNs retransmitted with different sequence numbers */
  /*  - FINs retransmitted with different sequence numbers */
  Bool bad_behavior;

  /* added for initial window stats (for Mallman) */
  u_int initialwin_bytes;	/* initial window (in bytes) */
  u_int initialwin_segs;	/* initial window (in segments) */
  Bool data_acked;		/* has any non-SYN data been acked? */

  /* added for (estimated) congestions window stats (for Mallman) */
  u_long cwin_max;
  u_long cwin_min;

  /* RTT stats for singly-transmitted segments */
  double rtt_last;		/* RTT as of last good ACK (microseconds) */
  u_long rtt_min;
  u_long rtt_max;
  double rtt_sum;		/* for averages */
  double rtt_sum2;		/* sum of squares, for stdev */
  double srtt;			/* smoothed RTT estimation */
  double rttvar;		/* smoothed stdv estimation */
  u_int rtt_count;		/* for averages */
/* MGM */
  /* Duplicated and rtx counters */
  u_int rtx_RTO;
  u_int rtx_FR;
  u_int reordering;
  u_int net_dup;
  u_int unknown;
  u_int flow_control;
  u_int unnecessary_rtx_FR;
  u_int unnecessary_rtx_RTO;
    /*LM*/ u_int ttl_min;
  u_int ttl_max;

  /* ACK Counters */
  u_long rtt_cumack;		/* segments only cumulativly ACKed */
  u_long rtt_nosample;		/* segments ACKED, but after retransmission */
  /* of earlier segments, so sample isn't */
  /* valid */
  u_long rtt_unkack;		/* unknown ACKs  ??? */
  u_long rtt_dupack;		/* duplicate ACKs */
  u_long rtt_triple_dupack;	/* triple duplicate ACKs */
  /* retransmission information */
  seqspace *ss;			/* the sequence space */
  u_long retr_max;		/* maximum retransmissions ct */
  u_long retr_min_tm;		/* minimum retransmissions time */
  u_long retr_max_tm;		/* maximum retransmissions time */
  double retr_tm_sum;		/* for averages */
  double retr_tm_sum2;		/* sum of squares, for stdev */
  u_long retr_tm_count;		/* for averages */

  /* Instantaneous throughput info */
  timeval thru_firsttime;	/* time of first packet this interval */
  u_long thru_bytes;		/* number of bytes this interval */
  u_long thru_pkts;		/* number of packets this interval */
  timeval thru_lasttime;	/* time of previous segment */

  /* data transfer time stamps - mallman */
  timeval first_data_time;
  timeval last_data_time;


  /* for tracking unidirectional idle time */
  timeval last_time;		/* last packet SENT from this side */
  u_llong idle_max;		/* maximum idle time observed (usecs) */

}
tcb;


typedef u_long hash;

typedef struct
{
  ipaddr a_address;
  ipaddr b_address;
  portnum a_port;
  portnum b_port;
  hash hash;
}
tcp_pair_addrblock;


struct stcp_pair
{

  /* endpoint identification */
  tcp_pair_addrblock addr_pair;

  /* connection naming information */
  Bool internal_src;
  Bool internal_dst;

  /* connection information */
  timeval first_time;
  timeval last_time;
  u_long packets;
  tcb c2s;
  tcb s2c;
};
typedef struct stcp_pair tcp_pair;

typedef struct tcphdr tcphdr;

typedef struct ptp_snap
{
  tcp_pair_addrblock addr_pair;	/* just a copy */
  struct ptp_snap *next;
  tcp_pair *ptp;
  tcp_pair **ttp_ptr;
}
ptp_snap;

extern int num_tcp_pairs;	/* how many pairs are in use */
extern tcp_pair **ttp;		/* array of pointers to allocated pairs */

enum tipo
{ unknown, first_RTP, first_RTCP, RTP, RTCP };	/* What UDP payload is in
						   this flow */
#define WIN 20			/* sliding window size used to track RTP
				   flows */
#define OVERFLOW_TH 2000	/* threshold to consider rtp segment as out
				   of sequence */

typedef struct rtp
{

  u_int16_t packets_win[WIN];	/* the sliding window vector used to track
				   seqnum */
  Bool w;			/* true if we got a full window */
  u_int16_t initial_seqno;
  u_int16_t largest_seqno;
  u_long pnum;			/* number of segments */
  u_int32_t ssrc;
  timeval first_time;
  timeval last_time;
  long int n_out_of_sequence;
  double sum_delta_t;
  int n_delta_t;
  int transit;
  u_int32_t first_ts;
  u_int32_t largest_ts;
  double jitter;
  long int n_dup;
  long int n_late;
  long int n_lost;
  int burst;
} rtp;


typedef struct rtcp
{
  u_long pnum;			/* number of segments */
  double sum_delta_t;		/* total interarrival time so far */
  timeval first_time;
  timeval last_time;
} rtcp;

/* minimal support for UDP "connections" */
typedef struct ucb
{
  /* parent pointer */
  struct sudp_pair *pup;
  struct ucb *ptwin;

  timeval time;
  enum tipo type;

  union
  {
    rtp rtp;
    rtcp rtcp;
  } flow;
  /* statistics added */
  u_llong data_bytes;
  u_llong packets;
}
ucb;


typedef tcp_pair_addrblock udp_pair_addrblock;
struct sudp_pair
{
  /* endpoint identification */
  udp_pair_addrblock addr_pair;

  /* connection naming information */
  Bool internal_src;
  Bool internal_dst;

  /* connection information */
  timeval first_time;
  timeval last_time;
  u_llong packets;
  ucb c2s;
  ucb s2c;

  /* linked list of usage */
  struct sudp_pair *next;
};
typedef struct sudp_pair udp_pair;
typedef struct udphdr udphdr;

#ifdef HAVE_RRDTOOL
/*-----------------------------------------------------------*/
/* RRDtools 				                     */

struct rrdconf
{
  Bool rrd;			/* boolean: whether to use rrd on this histo */
  Bool avg, min, max, var, stdev, idxoth, hit;	/* statistical var to dump */
  int *idx;			/* indexes to dump */
  int idxno;
  double *prc;			/* percentiles to dump */
  int prcno;
};

struct rrdstat
{
  double avg, min, max, pseudovar;	/* actual values of the statistical var to dump */
  long count;
  timeval last;
};
/*-----------------------------------------------------------*/
#endif
